"""
Python module generated from Java source file java.util.concurrent.Executors

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.security import AccessControlContext
from java.security import AccessControlException
from java.security import AccessController
from java.security import PrivilegedAction
from java.security import PrivilegedActionException
from java.security import PrivilegedExceptionAction
from java.util.concurrent import *
from java.util.concurrent.atomic import AtomicInteger
from sun.security.util import SecurityConstants
from typing import Any, Callable, Iterable, Tuple


class Executors:
    """
    Factory and utility methods for Executor, ExecutorService, ScheduledExecutorService, ThreadFactory, and Callable classes defined in this
    package. This class supports the following kinds of methods:
    
    
      - Methods that create and return an ExecutorService
          set up with commonly useful configuration settings.
      - Methods that create and return a ScheduledExecutorService
          set up with commonly useful configuration settings.
      - Methods that create and return a "wrapped" ExecutorService, that
          disables reconfiguration by making implementation-specific methods
          inaccessible.
      - Methods that create and return a ThreadFactory
          that sets newly created threads to a known state.
      - Methods that create and return a Callable
          out of other closure-like forms, so they can be used
          in execution methods requiring `Callable`.

    Author(s)
    - Doug Lea

    Since
    - 1.5
    """

    @staticmethod
    def newFixedThreadPool(nThreads: int) -> "ExecutorService":
        """
        Creates a thread pool that reuses a fixed number of threads
        operating off a shared unbounded queue.  At any point, at most
        `nThreads` threads will be active processing tasks.
        If additional tasks are submitted when all threads are active,
        they will wait in the queue until a thread is available.
        If any thread terminates due to a failure during execution
        prior to shutdown, a new one will take its place if needed to
        execute subsequent tasks.  The threads in the pool will exist
        until it is explicitly ExecutorService.shutdown shutdown.

        Arguments
        - nThreads: the number of threads in the pool

        Returns
        - the newly created thread pool

        Raises
        - IllegalArgumentException: if `nThreads <= 0`
        """
        ...


    @staticmethod
    def newWorkStealingPool(parallelism: int) -> "ExecutorService":
        """
        Creates a thread pool that maintains enough threads to support
        the given parallelism level, and may use multiple queues to
        reduce contention. The parallelism level corresponds to the
        maximum number of threads actively engaged in, or available to
        engage in, task processing. The actual number of threads may
        grow and shrink dynamically. A work-stealing pool makes no
        guarantees about the order in which submitted tasks are
        executed.

        Arguments
        - parallelism: the targeted parallelism level

        Returns
        - the newly created thread pool

        Raises
        - IllegalArgumentException: if `parallelism <= 0`

        Since
        - 1.8
        """
        ...


    @staticmethod
    def newWorkStealingPool() -> "ExecutorService":
        """
        Creates a work-stealing thread pool using the number of
        Runtime.availableProcessors available processors
        as its target parallelism level.

        Returns
        - the newly created thread pool

        See
        - .newWorkStealingPool(int)

        Since
        - 1.8
        """
        ...


    @staticmethod
    def newFixedThreadPool(nThreads: int, threadFactory: "ThreadFactory") -> "ExecutorService":
        """
        Creates a thread pool that reuses a fixed number of threads
        operating off a shared unbounded queue, using the provided
        ThreadFactory to create new threads when needed.  At any point,
        at most `nThreads` threads will be active processing
        tasks.  If additional tasks are submitted when all threads are
        active, they will wait in the queue until a thread is
        available.  If any thread terminates due to a failure during
        execution prior to shutdown, a new one will take its place if
        needed to execute subsequent tasks.  The threads in the pool will
        exist until it is explicitly ExecutorService.shutdown
        shutdown.

        Arguments
        - nThreads: the number of threads in the pool
        - threadFactory: the factory to use when creating new threads

        Returns
        - the newly created thread pool

        Raises
        - NullPointerException: if threadFactory is null
        - IllegalArgumentException: if `nThreads <= 0`
        """
        ...


    @staticmethod
    def newSingleThreadExecutor() -> "ExecutorService":
        """
        Creates an Executor that uses a single worker thread operating
        off an unbounded queue. (Note however that if this single
        thread terminates due to a failure during execution prior to
        shutdown, a new one will take its place if needed to execute
        subsequent tasks.)  Tasks are guaranteed to execute
        sequentially, and no more than one task will be active at any
        given time. Unlike the otherwise equivalent
        `newFixedThreadPool(1)` the returned executor is
        guaranteed not to be reconfigurable to use additional threads.

        Returns
        - the newly created single-threaded Executor
        """
        ...


    @staticmethod
    def newSingleThreadExecutor(threadFactory: "ThreadFactory") -> "ExecutorService":
        """
        Creates an Executor that uses a single worker thread operating
        off an unbounded queue, and uses the provided ThreadFactory to
        create a new thread when needed. Unlike the otherwise
        equivalent `newFixedThreadPool(1, threadFactory)` the
        returned executor is guaranteed not to be reconfigurable to use
        additional threads.

        Arguments
        - threadFactory: the factory to use when creating new threads

        Returns
        - the newly created single-threaded Executor

        Raises
        - NullPointerException: if threadFactory is null
        """
        ...


    @staticmethod
    def newCachedThreadPool() -> "ExecutorService":
        """
        Creates a thread pool that creates new threads as needed, but
        will reuse previously constructed threads when they are
        available.  These pools will typically improve the performance
        of programs that execute many short-lived asynchronous tasks.
        Calls to `execute` will reuse previously constructed
        threads if available. If no existing thread is available, a new
        thread will be created and added to the pool. Threads that have
        not been used for sixty seconds are terminated and removed from
        the cache. Thus, a pool that remains idle for long enough will
        not consume any resources. Note that pools with similar
        properties but different details (for example, timeout parameters)
        may be created using ThreadPoolExecutor constructors.

        Returns
        - the newly created thread pool
        """
        ...


    @staticmethod
    def newCachedThreadPool(threadFactory: "ThreadFactory") -> "ExecutorService":
        """
        Creates a thread pool that creates new threads as needed, but
        will reuse previously constructed threads when they are
        available, and uses the provided
        ThreadFactory to create new threads when needed.

        Arguments
        - threadFactory: the factory to use when creating new threads

        Returns
        - the newly created thread pool

        Raises
        - NullPointerException: if threadFactory is null
        """
        ...


    @staticmethod
    def newSingleThreadScheduledExecutor() -> "ScheduledExecutorService":
        """
        Creates a single-threaded executor that can schedule commands
        to run after a given delay, or to execute periodically.
        (Note however that if this single
        thread terminates due to a failure during execution prior to
        shutdown, a new one will take its place if needed to execute
        subsequent tasks.)  Tasks are guaranteed to execute
        sequentially, and no more than one task will be active at any
        given time. Unlike the otherwise equivalent
        `newScheduledThreadPool(1)` the returned executor is
        guaranteed not to be reconfigurable to use additional threads.

        Returns
        - the newly created scheduled executor
        """
        ...


    @staticmethod
    def newSingleThreadScheduledExecutor(threadFactory: "ThreadFactory") -> "ScheduledExecutorService":
        """
        Creates a single-threaded executor that can schedule commands
        to run after a given delay, or to execute periodically.  (Note
        however that if this single thread terminates due to a failure
        during execution prior to shutdown, a new one will take its
        place if needed to execute subsequent tasks.)  Tasks are
        guaranteed to execute sequentially, and no more than one task
        will be active at any given time. Unlike the otherwise
        equivalent `newScheduledThreadPool(1, threadFactory)`
        the returned executor is guaranteed not to be reconfigurable to
        use additional threads.

        Arguments
        - threadFactory: the factory to use when creating new threads

        Returns
        - the newly created scheduled executor

        Raises
        - NullPointerException: if threadFactory is null
        """
        ...


    @staticmethod
    def newScheduledThreadPool(corePoolSize: int) -> "ScheduledExecutorService":
        """
        Creates a thread pool that can schedule commands to run after a
        given delay, or to execute periodically.

        Arguments
        - corePoolSize: the number of threads to keep in the pool,
        even if they are idle

        Returns
        - the newly created scheduled thread pool

        Raises
        - IllegalArgumentException: if `corePoolSize < 0`
        """
        ...


    @staticmethod
    def newScheduledThreadPool(corePoolSize: int, threadFactory: "ThreadFactory") -> "ScheduledExecutorService":
        """
        Creates a thread pool that can schedule commands to run after a
        given delay, or to execute periodically.

        Arguments
        - corePoolSize: the number of threads to keep in the pool,
        even if they are idle
        - threadFactory: the factory to use when the executor
        creates a new thread

        Returns
        - the newly created scheduled thread pool

        Raises
        - IllegalArgumentException: if `corePoolSize < 0`
        - NullPointerException: if threadFactory is null
        """
        ...


    @staticmethod
    def unconfigurableExecutorService(executor: "ExecutorService") -> "ExecutorService":
        """
        Returns an object that delegates all defined ExecutorService methods to the given executor, but not any
        other methods that might otherwise be accessible using
        casts. This provides a way to safely "freeze" configuration and
        disallow tuning of a given concrete implementation.

        Arguments
        - executor: the underlying implementation

        Returns
        - an `ExecutorService` instance

        Raises
        - NullPointerException: if executor null
        """
        ...


    @staticmethod
    def unconfigurableScheduledExecutorService(executor: "ScheduledExecutorService") -> "ScheduledExecutorService":
        """
        Returns an object that delegates all defined ScheduledExecutorService methods to the given executor, but
        not any other methods that might otherwise be accessible using
        casts. This provides a way to safely "freeze" configuration and
        disallow tuning of a given concrete implementation.

        Arguments
        - executor: the underlying implementation

        Returns
        - a `ScheduledExecutorService` instance

        Raises
        - NullPointerException: if executor null
        """
        ...


    @staticmethod
    def defaultThreadFactory() -> "ThreadFactory":
        """
        Returns a default thread factory used to create new threads.
        This factory creates all new threads used by an Executor in the
        same ThreadGroup. If there is a java.lang.SecurityManager, it uses the group of System.getSecurityManager, else the group of the thread
        invoking this `defaultThreadFactory` method. Each new
        thread is created as a non-daemon thread with priority set to
        the smaller of `Thread.NORM_PRIORITY` and the maximum
        priority permitted in the thread group.  New threads have names
        accessible via Thread.getName of
        *pool-N-thread-M*, where *N* is the sequence
        number of this factory, and *M* is the sequence number
        of the thread created by this factory.

        Returns
        - a thread factory
        """
        ...


    @staticmethod
    def privilegedThreadFactory() -> "ThreadFactory":
        """
        Returns a thread factory used to create new threads that
        have the same permissions as the current thread.
        This factory creates threads with the same settings as Executors.defaultThreadFactory, additionally setting the
        AccessControlContext and contextClassLoader of new threads to
        be the same as the thread invoking this
        `privilegedThreadFactory` method.  A new
        `privilegedThreadFactory` can be created within an
        AccessController.doPrivileged AccessController.doPrivileged
        action setting the current thread's access control context to
        create threads with the selected permission settings holding
        within that action.
        
        Note that while tasks running within such threads will have
        the same access control and class loader settings as the
        current thread, they need not have the same java.lang.ThreadLocal or java.lang.InheritableThreadLocal values. If necessary,
        particular values of thread locals can be set or reset before
        any task runs in ThreadPoolExecutor subclasses using
        ThreadPoolExecutor.beforeExecute(Thread, Runnable).
        Also, if it is necessary to initialize worker threads to have
        the same InheritableThreadLocal settings as some other
        designated thread, you can create a custom ThreadFactory in
        which that thread waits for and services requests to create
        others that will inherit its values.

        Returns
        - a thread factory

        Raises
        - AccessControlException: if the current access control
        context does not have permission to both get and set context
        class loader

        Deprecated
        - This method is only useful in conjunction with
              SecurityManager the Security Manager, which is
              deprecated and subject to removal in a future release.
              Consequently, this method is also deprecated and subject to
              removal. There is no replacement for the Security Manager or this
              method.
        """
        ...


    @staticmethod
    def callable(task: "Runnable", result: "T") -> "Callable"["T"]:
        """
        Returns a Callable object that, when
        called, runs the given task and returns the given result.  This
        can be useful when applying methods requiring a
        `Callable` to an otherwise resultless action.
        
        Type `<T>`: the type of the result

        Arguments
        - task: the task to run
        - result: the result to return

        Returns
        - a callable object

        Raises
        - NullPointerException: if task null
        """
        ...


    @staticmethod
    def callable(task: "Runnable") -> "Callable"["Object"]:
        """
        Returns a Callable object that, when
        called, runs the given task and returns `null`.

        Arguments
        - task: the task to run

        Returns
        - a callable object

        Raises
        - NullPointerException: if task null
        """
        ...


    @staticmethod
    def callable(action: "PrivilegedAction"[Any]) -> "Callable"["Object"]:
        """
        Returns a Callable object that, when
        called, runs the given privileged action and returns its result.

        Arguments
        - action: the privileged action to run

        Returns
        - a callable object

        Raises
        - NullPointerException: if action null
        """
        ...


    @staticmethod
    def callable(action: "PrivilegedExceptionAction"[Any]) -> "Callable"["Object"]:
        """
        Returns a Callable object that, when
        called, runs the given privileged exception action and returns
        its result.

        Arguments
        - action: the privileged exception action to run

        Returns
        - a callable object

        Raises
        - NullPointerException: if action null
        """
        ...


    @staticmethod
    def privilegedCallable(callable: "Callable"["T"]) -> "Callable"["T"]:
        """
        Returns a Callable object that will, when called,
        execute the given `callable` under the current access
        control context. This method should normally be invoked within
        an AccessController.doPrivileged AccessController.doPrivileged
        action to create callables that will, if possible, execute
        under the selected permission settings holding within that
        action; or if not possible, throw an associated AccessControlException.
        
        Type `<T>`: the type of the callable's result

        Arguments
        - callable: the underlying task

        Returns
        - a callable object

        Raises
        - NullPointerException: if callable null

        Deprecated
        - This method is only useful in conjunction with
              SecurityManager the Security Manager, which is
              deprecated and subject to removal in a future release.
              Consequently, this method is also deprecated and subject to
              removal. There is no replacement for the Security Manager or this
              method.
        """
        ...


    @staticmethod
    def privilegedCallableUsingCurrentClassLoader(callable: "Callable"["T"]) -> "Callable"["T"]:
        """
        Returns a Callable object that will, when called,
        execute the given `callable` under the current access
        control context, with the current context class loader as the
        context class loader. This method should normally be invoked
        within an
        AccessController.doPrivileged AccessController.doPrivileged
        action to create callables that will, if possible, execute
        under the selected permission settings holding within that
        action; or if not possible, throw an associated AccessControlException.
        
        Type `<T>`: the type of the callable's result

        Arguments
        - callable: the underlying task

        Returns
        - a callable object

        Raises
        - NullPointerException: if callable null
        - AccessControlException: if the current access control
        context does not have permission to both set and get context
        class loader

        Deprecated
        - This method is only useful in conjunction with
              SecurityManager the Security Manager, which is
              deprecated and subject to removal in a future release.
              Consequently, this method is also deprecated and subject to
              removal. There is no replacement for the Security Manager or this
              method.
        """
        ...



"""
Python module generated from Java source file com.google.common.collect.ForwardingListIterator

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.collect import *
from com.google.errorprone.annotations import CanIgnoreReturnValue
from java.util import ListIterator
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class ForwardingListIterator(ForwardingIterator, ListIterator):
    """
    A list iterator which forwards all its method calls to another list iterator. Subclasses should
    override one or more methods to modify the behavior of the backing iterator as desired per the <a
    href="http://en.wikipedia.org/wiki/Decorator_pattern">decorator pattern</a>.
    
    **`default` method warning:** This class forwards calls to *only some* `default` methods. Specifically, it forwards calls only for methods that existed <a
    href="https://docs.oracle.com/javase/7/docs/api/java/util/ListIterator.html">before `default` methods were introduced</a>. For newer methods, like `forEachRemaining`, it
    inherits their default implementations. When those implementations invoke methods, they invoke
    methods on the `ForwardingListIterator`.

    Author(s)
    - Mike Bostock

    Since
    - 2.0
    """

    def add(self, element: "E") -> None:
        ...


    def hasPrevious(self) -> bool:
        ...


    def nextIndex(self) -> int:
        ...


    def previous(self) -> "E":
        ...


    def previousIndex(self) -> int:
        ...


    def set(self, element: "E") -> None:
        ...



"""
Python module generated from Java source file java.io.OutputStreamWriter

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.io import *
from java.nio.charset import Charset
from java.nio.charset import CharsetEncoder
from sun.nio.cs import StreamEncoder
from typing import Any, Callable, Iterable, Tuple


class OutputStreamWriter(Writer):

    def __init__(self, out: "OutputStream", charsetName: str):
        """
        Creates an OutputStreamWriter that uses the named charset.

        Arguments
        - out: An OutputStream
        - charsetName: The name of a supported
                java.nio.charset.Charset charset

        Raises
        - UnsupportedEncodingException: If the named encoding is not supported
        """
        ...


    def __init__(self, out: "OutputStream"):
        """
        Creates an OutputStreamWriter that uses the default character encoding.

        Arguments
        - out: An OutputStream
        """
        ...


    def __init__(self, out: "OutputStream", cs: "Charset"):
        """
        Creates an OutputStreamWriter that uses the given charset.

        Arguments
        - out: An OutputStream
        - cs: A charset

        Since
        - 1.4
        """
        ...


    def __init__(self, out: "OutputStream", enc: "CharsetEncoder"):
        """
        Creates an OutputStreamWriter that uses the given charset encoder.

        Arguments
        - out: An OutputStream
        - enc: A charset encoder

        Since
        - 1.4
        """
        ...


    def getEncoding(self) -> str:
        """
        Returns the name of the character encoding being used by this stream.
        
         If the encoding has an historical name then that name is returned;
        otherwise the encoding's canonical name is returned.
        
         If this instance was created with the .OutputStreamWriter(OutputStream, String) constructor then the returned
        name, being unique for the encoding, may differ from the name passed to
        the constructor.  This method may return `null` if the stream has
        been closed. 

        Returns
        - The historical name of this encoding, or possibly
                `null` if the stream has been closed

        See
        - java.nio.charset.Charset

        Unknown Tags
        - 1.4
        """
        ...


    def write(self, c: int) -> None:
        """
        Writes a single character.

        Raises
        - IOException: If an I/O error occurs
        """
        ...


    def write(self, cbuf: list[str], off: int, len: int) -> None:
        """
        Writes a portion of an array of characters.

        Arguments
        - cbuf: Buffer of characters
        - off: Offset from which to start writing characters
        - len: Number of characters to write

        Raises
        - IndexOutOfBoundsException: If `off` is negative, or `len` is negative,
                 or `off + len` is negative or greater than the length
                 of the given array
        - IOException: If an I/O error occurs
        """
        ...


    def write(self, str: str, off: int, len: int) -> None:
        """
        Writes a portion of a string.

        Arguments
        - str: A String
        - off: Offset from which to start writing characters
        - len: Number of characters to write

        Raises
        - IndexOutOfBoundsException: If `off` is negative, or `len` is negative,
                 or `off + len` is negative or greater than the length
                 of the given string
        - IOException: If an I/O error occurs
        """
        ...


    def append(self, csq: "CharSequence", start: int, end: int) -> "Writer":
        ...


    def append(self, csq: "CharSequence") -> "Writer":
        ...


    def flush(self) -> None:
        """
        Flushes the stream.

        Raises
        - IOException: If an I/O error occurs
        """
        ...


    def close(self) -> None:
        ...



"""
Python module generated from Java source file com.google.gson.FieldNamingStrategy

Java source file obtained from artifact gson version 2.11.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.gson import *
from java.lang.reflect import Field
from typing import Any, Callable, Iterable, Tuple


class FieldNamingStrategy:
    """
    A mechanism for providing custom field naming in Gson. This allows the client code to translate
    field names into a particular convention that is not supported as a normal Java field declaration
    rules. For example, Java does not support "-" characters in a field name.

    Author(s)
    - Joel Leitch

    Since
    - 1.3
    """

    def translateName(self, f: "Field") -> str:
        """
        Translates the field name into its JSON field name representation.

        Arguments
        - f: the field object that we are translating

        Returns
        - the translated field name.

        Since
        - 1.3
        """
        ...



"""
Python module generated from Java source file com.google.common.util.concurrent.ServiceManager

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.base import Function
from com.google.common.base import MoreObjects
from com.google.common.base import Stopwatch
from com.google.common.collect import Collections2
from com.google.common.collect import ImmutableCollection
from com.google.common.collect import ImmutableList
from com.google.common.collect import ImmutableMap
from com.google.common.collect import ImmutableSet
from com.google.common.collect import ImmutableSetMultimap
from com.google.common.collect import Lists
from com.google.common.collect import Maps
from com.google.common.collect import MultimapBuilder
from com.google.common.collect import Multimaps
from com.google.common.collect import Multiset
from com.google.common.collect import Ordering
from com.google.common.collect import SetMultimap
from com.google.common.util.concurrent import *
from com.google.common.util.concurrent.Service import State
from com.google.errorprone.annotations import CanIgnoreReturnValue
from com.google.errorprone.annotations.concurrent import GuardedBy
from com.google.j2objc.annotations import J2ObjCIncompatible
from com.google.j2objc.annotations import WeakOuter
from java.lang.ref import WeakReference
from java.time import Duration
from java.util import Collections
from java.util import EnumSet
from java.util import IdentityHashMap
from java.util.concurrent import Executor
from java.util.concurrent import TimeUnit
from java.util.concurrent import TimeoutException
from typing import Any, Callable, Iterable, Tuple


class ServiceManager(ServiceManagerBridge):
    """
    A manager for monitoring and controlling a set of Service services. This class
    provides methods for .startAsync() starting, .stopAsync() stopping and
    .servicesByState inspecting a collection of Service services.
    Additionally, users can monitor state transitions with the Listener listener
    mechanism.
    
    While it is recommended that service lifecycles be managed via this class, state transitions
    initiated via other mechanisms do not impact the correctness of its methods. For example, if the
    services are started by some mechanism besides .startAsync, the listeners will be invoked
    when appropriate and .awaitHealthy will still work as expected.
    
    Here is a simple example of how to use a `ServiceManager` to start a server.
    
    ````class Server {
      public static void main(String[] args) {
        Set<Service> services = ...;
        ServiceManager manager = new ServiceManager(services);
        manager.addListener(new Listener() {
            public void stopped() {`
            public void healthy() {
              // Services have been initialized and are healthy, start accepting requests...
            }
            public void failure(Service service) {
              // Something failed, at this point we could log it, notify a load balancer, or take
              // some other action.  For now we will just exit.
              System.exit(1);
            }
          },
          MoreExecutors.directExecutor());
    
        Runtime.getRuntime().addShutdownHook(new Thread() {
          public void run() {
            // Give the services 5 seconds to stop to ensure that we are responsive to shutdown
            // requests.
            try {
              manager.stopAsync().awaitStopped(5, TimeUnit.SECONDS);
            } catch (TimeoutException timeout) {
              // stopping timed out
            }
          }
        });
        manager.startAsync();  // start all the services asynchronously
      }
    }
    }```
    
    This class uses the ServiceManager's methods to start all of its services, to respond to
    service failure and to ensure that when the JVM is shutting down all the services are stopped.

    Author(s)
    - Luke Sandberg

    Since
    - 14.0
    """

    def __init__(self, services: Iterable["Service"]):
        """
        Constructs a new instance for managing the given services.

        Arguments
        - services: The services to manage

        Raises
        - IllegalArgumentException: if not all services are State.NEW new or if there
            are any duplicate services.
        """
        ...


    def addListener(self, listener: "Listener", executor: "Executor") -> None:
        """
        Registers a Listener to be Executor.execute executed on the given
        executor. The listener will not have previous state changes replayed, so it is suggested that
        listeners are added before any of the managed services are Service.startAsync
        started.
        
        `addListener` guarantees execution ordering across calls to a given listener but not
        across calls to multiple listeners. Specifically, a given listener will have its callbacks
        invoked in the same order as the underlying service enters those states. Additionally, at most
        one of the listener's callbacks will execute at once. However, multiple listeners' callbacks
        may execute concurrently, and listeners may execute in an order different from the one in which
        they were registered.
        
        RuntimeExceptions thrown by a listener will be caught and logged. Any exception thrown
        during `Executor.execute` (e.g., a `RejectedExecutionException`) will be caught and
        logged.
        
        When selecting an executor, note that `directExecutor` is dangerous in some cases. See
        the discussion in the ListenableFuture.addListener ListenableFuture.addListener
        documentation.

        Arguments
        - listener: the listener to run when the manager changes state
        - executor: the executor in which the listeners callback methods will be run.
        """
        ...


    def startAsync(self) -> "ServiceManager":
        """
        Initiates service Service.startAsync startup on all the services being managed. It
        is only valid to call this method if all of the services are State.NEW new.

        Returns
        - this

        Raises
        - IllegalStateException: if any of the Services are not State.NEW new when the
            method is called.
        """
        ...


    def awaitHealthy(self) -> None:
        """
        Waits for the ServiceManager to become .isHealthy() healthy. The manager
        will become healthy after all the component services have reached the State.RUNNING
        running state.

        Raises
        - IllegalStateException: if the service manager reaches a state from which it cannot
            become .isHealthy() healthy.
        """
        ...


    def awaitHealthy(self, timeout: "Duration") -> None:
        """
        Waits for the ServiceManager to become .isHealthy() healthy for no more
        than the given time. The manager will become healthy after all the component services have
        reached the State.RUNNING running state.

        Arguments
        - timeout: the maximum time to wait

        Raises
        - TimeoutException: if not all of the services have finished starting within the deadline
        - IllegalStateException: if the service manager reaches a state from which it cannot
            become .isHealthy() healthy.

        Since
        - 28.0
        """
        ...


    def awaitHealthy(self, timeout: int, unit: "TimeUnit") -> None:
        """
        Waits for the ServiceManager to become .isHealthy() healthy for no more
        than the given time. The manager will become healthy after all the component services have
        reached the State.RUNNING running state.

        Arguments
        - timeout: the maximum time to wait
        - unit: the time unit of the timeout argument

        Raises
        - TimeoutException: if not all of the services have finished starting within the deadline
        - IllegalStateException: if the service manager reaches a state from which it cannot
            become .isHealthy() healthy.
        """
        ...


    def stopAsync(self) -> "ServiceManager":
        """
        Initiates service Service.stopAsync shutdown if necessary on all the services
        being managed.

        Returns
        - this
        """
        ...


    def awaitStopped(self) -> None:
        """
        Waits for the all the services to reach a terminal state. After this method returns all
        services will either be Service.State.TERMINATED terminated or Service.State.FAILED failed.
        """
        ...


    def awaitStopped(self, timeout: "Duration") -> None:
        """
        Waits for the all the services to reach a terminal state for no more than the given time. After
        this method returns all services will either be Service.State.TERMINATED
        terminated or Service.State.FAILED failed.

        Arguments
        - timeout: the maximum time to wait

        Raises
        - TimeoutException: if not all of the services have stopped within the deadline

        Since
        - 28.0
        """
        ...


    def awaitStopped(self, timeout: int, unit: "TimeUnit") -> None:
        """
        Waits for the all the services to reach a terminal state for no more than the given time. After
        this method returns all services will either be Service.State.TERMINATED
        terminated or Service.State.FAILED failed.

        Arguments
        - timeout: the maximum time to wait
        - unit: the time unit of the timeout argument

        Raises
        - TimeoutException: if not all of the services have stopped within the deadline
        """
        ...


    def isHealthy(self) -> bool:
        """
        Returns True if all services are currently in the State.RUNNING running state.
        
        Users who want more detailed information should use the .servicesByState method to
        get detailed information about which services are not running.
        """
        ...


    def servicesByState(self) -> "ImmutableSetMultimap"["State", "Service"]:
        """
        Provides a snapshot of the current state of all the services under management.
        
        N.B. This snapshot is guaranteed to be consistent, i.e. the set of states returned will
        correspond to a point in time view of the services.

        Since
        - 29.0 (present with return type `ImmutableMultimap` since 14.0)
        """
        ...


    def startupTimes(self) -> "ImmutableMap"["Service", "Long"]:
        """
        Returns the service load times. This value will only return startup times for services that
        have finished starting.

        Returns
        - Map of services and their corresponding startup time in millis, the map entries will be
            ordered by startup time.
        """
        ...


    def startupDurations(self) -> "ImmutableMap"["Service", "Duration"]:
        """
        Returns the service load times. This value will only return startup times for services that
        have finished starting.

        Returns
        - Map of services and their corresponding startup time, the map entries will be ordered
            by startup time.

        Since
        - 31.0
        """
        ...


    def toString(self) -> str:
        ...


    class Listener:
        """
        A listener for the aggregate state changes of the services that are under management. Users
        that need to listen to more fine-grained events (such as when each particular Service service starts, or terminates), should attach Service.Listener service
        listeners to each individual service.

    Author(s)
        - Luke Sandberg

        Since
        - 15.0 (present as an interface in 14.0)
        """

        def healthy(self) -> None:
            """
            Called when the service initially becomes healthy.
            
            This will be called at most once after all the services have entered the State.RUNNING running state. If any services fail during start up or State.FAILED fail/State.TERMINATED terminate before all other services have
            started State.RUNNING running then this method will not be called.
            """
            ...


        def stopped(self) -> None:
            """
            Called when the all of the component services have reached a terminal state, either
            State.TERMINATED terminated or State.FAILED failed.
            """
            ...


        def failure(self, service: "Service") -> None:
            """
            Called when a component service has State.FAILED failed.

            Arguments
            - service: The service that failed.
            """
            ...



"""
Python module generated from Java source file com.google.gson.JsonSerializer

Java source file obtained from artifact gson version 2.11.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.gson import *
from java.lang.reflect import Type
from typing import Any, Callable, Iterable, Tuple


class JsonSerializer:
    """
    Interface representing a custom serializer for JSON. You should write a custom serializer, if you
    are not happy with the default serialization done by Gson. You will also need to register this
    serializer through com.google.gson.GsonBuilder.registerTypeAdapter(Type, Object).
    
    Let us look at example where defining a serializer will be useful. The `Id` class
    defined below has two fields: `clazz` and `value`.
    
    ```
    public class Id&lt;T&gt; {
      private final Class&lt;T&gt; clazz;
      private final long value;
    
      public Id(Class&lt;T&gt; clazz, long value) {
        this.clazz = clazz;
        this.value = value;
      }
    
      public long getValue() {
        return value;
      }
    }
    ```
    
    The default serialization of `Id(com.foo.MyObject.class, 20L)` will be `
    {"clazz":"com.foo.MyObject","value":20}`. Suppose, you just want the output to be the value
    instead, which is `20` in this case. You can achieve that by writing a custom serializer:
    
    ```
    class IdSerializer implements JsonSerializer&lt;Id&gt; {
      public JsonElement serialize(Id id, Type typeOfId, JsonSerializationContext context) {
        return new JsonPrimitive(id.getValue());
      }
    }
    ```
    
    You will also need to register `IdSerializer` with Gson as follows:
    
    ```
    Gson gson = new GsonBuilder().registerTypeAdapter(Id.class, new IdSerializer()).create();
    ```
    
    Serializers should be stateless and thread-safe, otherwise the thread-safety guarantees of
    Gson might not apply.
    
    New applications should prefer TypeAdapter, whose streaming API is more efficient than
    this interface's tree API.
    
    Type `<T>`: type for which the serializer is being registered. It is possible that a serializer
        may be asked to serialize a specific generic type of the T.

    Author(s)
    - Joel Leitch
    """

    def serialize(self, src: "T", typeOfSrc: "Type", context: "JsonSerializationContext") -> "JsonElement":
        """
        Gson invokes this call-back method during serialization when it encounters a field of the
        specified type.
        
        In the implementation of this call-back method, you should consider invoking JsonSerializationContext.serialize(Object, Type) method to create JsonElements for any
        non-trivial field of the `src` object. However, you should never invoke it on the `src` object itself since that will cause an infinite loop (Gson will call your call-back method
        again).

        Arguments
        - src: the object that needs to be converted to Json.
        - typeOfSrc: the actual type (fully genericized version) of the source object.

        Returns
        - a JsonElement corresponding to the specified object.
        """
        ...



"""
Python module generated from Java source file org.bukkit.entity.LightningStrike

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import GameEvent
from org.bukkit.enchantments import Enchantment
from org.bukkit.entity import *
from typing import Any, Callable, Iterable, Tuple


class LightningStrike(Entity):
    """
    Represents an instance of a lightning strike. May or may not do damage.
    """

    def isEffect(self) -> bool:
        """
        Returns whether the strike is an effect that does no damage.

        Returns
        - whether the strike is an effect
        """
        ...


    def getFlashes(self) -> int:
        """
        Get the amount of flashes that will occur before the lightning is
        removed. By default this value is between 1 and 3.

        Returns
        - the flashes
        """
        ...


    def setFlashes(self, flashes: int) -> None:
        """
        Set the amount of flashes that will occur before the lightning is
        removed. One flash will occur after this lightning strike's life
        has reduced below 0.

        Arguments
        - flashes: the flashes
        """
        ...


    def getLifeTicks(self) -> int:
        """
        Get the amount of ticks this lightning strike will inflict damage
        upon its hit entities.
        
        When life ticks are negative, there is a random chance that another
        flash will be initiated and life ticks reset to 1.

        Returns
        - the life ticks
        """
        ...


    def setLifeTicks(self, ticks: int) -> None:
        """
        Get the amount of ticks this lightning strike will inflict damage
        upon its hit entities.
        
        When life ticks are negative, there is a random chance that another
        flash will be initiated and life ticks reset to 1. Additionally, if
        life ticks are set to 2 (the default value when a lightning strike
        has been spawned), a list of events will occur:
        
          - Impact sound effects will be played
          - Fire will be spawned (dependent on difficulty)
          - Lightning rods will be powered (if hit)
          - Copper will be stripped (if hit)
          - GameEvent.LIGHTNING_STRIKE will be dispatched

        Arguments
        - ticks: the life ticks
        """
        ...


    def getCausingPlayer(self) -> "Player":
        """
        Get the Player that caused this lightning to strike. This
        will occur naturally if a trident enchanted with
        Enchantment.CHANNELING Channeling were thrown at an entity
        during a storm.

        Returns
        - the player
        """
        ...


    def setCausingPlayer(self, player: "Player") -> None:
        """
        Set the Player that caused this lightning to strike.

        Arguments
        - player: the player
        """
        ...


    def spigot(self) -> "Spigot":
        ...


    class Spigot(Spigot):

        def isSilent(self) -> bool:
            """
            Returns whether the strike is silent.

            Returns
            - whether the strike is silent.

            Deprecated
            - sound is now client side and cannot be removed
            """
            ...



from .UnsignedLongs import UnsignedLongs
from .UnsignedLong import UnsignedLong
from .UnsignedInts import UnsignedInts
from .UnsignedInteger import UnsignedInteger
from .UnsignedBytes import UnsignedBytes
from .SignedBytes import SignedBytes
from .ShortsMethodsForWeb import ShortsMethodsForWeb
from .Shorts import Shorts
from .Primitives import Primitives
from .ParseRequest import ParseRequest
from .ParametricNullness import ParametricNullness
from .Longs import Longs
from .IntsMethodsForWeb import IntsMethodsForWeb
from .Ints import Ints
from .ImmutableLongArray import ImmutableLongArray
from .ImmutableIntArray import ImmutableIntArray
from .ImmutableDoubleArray import ImmutableDoubleArray
from .FloatsMethodsForWeb import FloatsMethodsForWeb
from .Floats import Floats
from .ElementTypesAreNonnullByDefault import ElementTypesAreNonnullByDefault
from .DoublesMethodsForWeb import DoublesMethodsForWeb
from .Doubles import Doubles
from .Chars import Chars
from .Bytes import Bytes
from .Booleans import Booleans


"""
Python module generated from Java source file org.bukkit.inventory.view.LecternView

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.inventory import InventoryView
from org.bukkit.inventory import LecternInventory
from org.bukkit.inventory.view import *
from typing import Any, Callable, Iterable, Tuple


class LecternView(InventoryView):
    """
    An instance of InventoryView which provides extra methods related to
    lectern view data.
    """

    def getTopInventory(self) -> "LecternInventory":
        ...


    def getPage(self) -> int:
        """
        Gets the page that the LecternView is on.

        Returns
        - The page the book is on
        """
        ...


    def setPage(self, page: int) -> None:
        """
        Sets the page of the lectern book.

        Arguments
        - page: the lectern book page
        """
        ...



"""
Python module generated from Java source file org.bukkit.event.block.SculkBloomEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.base import Preconditions
from org.bukkit.block import Block
from org.bukkit.block import SculkCatalyst
from org.bukkit.event import Cancellable
from org.bukkit.event import HandlerList
from org.bukkit.event.block import *
from org.bukkit.event.entity import EntityDeathEvent
from typing import Any, Callable, Iterable, Tuple


class SculkBloomEvent(BlockEvent, Cancellable):
    """
    Represents an event triggered when a new cursor is created by a SculkCatalyst.
    
    <strong>Cursor Definition:</strong>
    A cursor in this context is a dynamic marker or pointer generated by the
    SculkCatalyst. It occupies a block and spreads sculk as it moves. It is
    similar to entity, but it is not an entity. Cursors are ticked by the
    tile entity.
    
    <strong>Triggers for Cursor Creation:</strong>
    
      - An entity, when killed and drops experience, within an 8-block radius of a SculkCatalyst.
      - An explicit call from a plugin using SculkCatalyst.bloom(Block, int).
    
    
    The result of .getBlock() is the location that the cursor is spawning at.
    """

    def __init__(self, theBlock: "Block", charge: int):
        ...


    def getCharge(self) -> int:
        """
        Returns the charge of the cursor, &lt; 1000 by default.

        Returns
        - the charge of the cursor
        """
        ...


    def setCharge(self, charge: int) -> None:
        """
        Sets the charge of the cursor.
        
        Increasing the charge of a cursor makes the cursor last longer, giving
        it more time to spread sculk blocks across a larger range.
        
        Typically, charges should be set to the exp reward of a mob
        (EntityDeathEvent.getDroppedExp()), which is usually
        3-5 for animals, and 5-10 for the average mob (up to 50 for
        wither skeletons). Roughly speaking, for each charge, 1 more
        sculk block will be placed.

        Arguments
        - charge: the charge of the cursor.
        """
        ...


    def isCancelled(self) -> bool:
        ...


    def setCancelled(self, cancel: bool) -> None:
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...



"""
Python module generated from Java source file java.lang.reflect.Modifier

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.lang.reflect import *
from java.util import StringJoiner
from typing import Any, Callable, Iterable, Tuple


class Modifier:
    """
    The Modifier class provides `static` methods and
    constants to decode class and member access modifiers.  The sets of
    modifiers are represented as integers with distinct bit positions
    representing different modifiers.  The values for the constants
    representing the modifiers are taken from the tables in sections
    4.1, 4.4, 4.5, and 4.7 of
    <cite>The Java Virtual Machine Specification</cite>.

    Author(s)
    - Kenneth Russell

    See
    - Member.getModifiers()

    Since
    - 1.1
    """

    PUBLIC = 0x00000001
    """
    The `int` value representing the `public`
    modifier.
    """
    PRIVATE = 0x00000002
    """
    The `int` value representing the `private`
    modifier.
    """
    PROTECTED = 0x00000004
    """
    The `int` value representing the `protected`
    modifier.
    """
    STATIC = 0x00000008
    """
    The `int` value representing the `static`
    modifier.
    """
    FINAL = 0x00000010
    """
    The `int` value representing the `final`
    modifier.
    """
    SYNCHRONIZED = 0x00000020
    """
    The `int` value representing the `synchronized`
    modifier.
    """
    VOLATILE = 0x00000040
    """
    The `int` value representing the `volatile`
    modifier.
    """
    TRANSIENT = 0x00000080
    """
    The `int` value representing the `transient`
    modifier.
    """
    NATIVE = 0x00000100
    """
    The `int` value representing the `native`
    modifier.
    """
    INTERFACE = 0x00000200
    """
    The `int` value representing the `interface`
    modifier.
    """
    ABSTRACT = 0x00000400
    """
    The `int` value representing the `abstract`
    modifier.
    """
    STRICT = 0x00000800
    """
    The `int` value representing the `strictfp`
    modifier.
    """


    @staticmethod
    def isPublic(mod: int) -> bool:
        """
        Return `True` if the integer argument includes the
        `public` modifier, `False` otherwise.

        Arguments
        - mod: a set of modifiers

        Returns
        - `True` if `mod` includes the
        `public` modifier; `False` otherwise.
        """
        ...


    @staticmethod
    def isPrivate(mod: int) -> bool:
        """
        Return `True` if the integer argument includes the
        `private` modifier, `False` otherwise.

        Arguments
        - mod: a set of modifiers

        Returns
        - `True` if `mod` includes the
        `private` modifier; `False` otherwise.
        """
        ...


    @staticmethod
    def isProtected(mod: int) -> bool:
        """
        Return `True` if the integer argument includes the
        `protected` modifier, `False` otherwise.

        Arguments
        - mod: a set of modifiers

        Returns
        - `True` if `mod` includes the
        `protected` modifier; `False` otherwise.
        """
        ...


    @staticmethod
    def isStatic(mod: int) -> bool:
        """
        Return `True` if the integer argument includes the
        `static` modifier, `False` otherwise.

        Arguments
        - mod: a set of modifiers

        Returns
        - `True` if `mod` includes the
        `static` modifier; `False` otherwise.
        """
        ...


    @staticmethod
    def isFinal(mod: int) -> bool:
        """
        Return `True` if the integer argument includes the
        `final` modifier, `False` otherwise.

        Arguments
        - mod: a set of modifiers

        Returns
        - `True` if `mod` includes the
        `final` modifier; `False` otherwise.
        """
        ...


    @staticmethod
    def isSynchronized(mod: int) -> bool:
        """
        Return `True` if the integer argument includes the
        `synchronized` modifier, `False` otherwise.

        Arguments
        - mod: a set of modifiers

        Returns
        - `True` if `mod` includes the
        `synchronized` modifier; `False` otherwise.
        """
        ...


    @staticmethod
    def isVolatile(mod: int) -> bool:
        """
        Return `True` if the integer argument includes the
        `volatile` modifier, `False` otherwise.

        Arguments
        - mod: a set of modifiers

        Returns
        - `True` if `mod` includes the
        `volatile` modifier; `False` otherwise.
        """
        ...


    @staticmethod
    def isTransient(mod: int) -> bool:
        """
        Return `True` if the integer argument includes the
        `transient` modifier, `False` otherwise.

        Arguments
        - mod: a set of modifiers

        Returns
        - `True` if `mod` includes the
        `transient` modifier; `False` otherwise.
        """
        ...


    @staticmethod
    def isNative(mod: int) -> bool:
        """
        Return `True` if the integer argument includes the
        `native` modifier, `False` otherwise.

        Arguments
        - mod: a set of modifiers

        Returns
        - `True` if `mod` includes the
        `native` modifier; `False` otherwise.
        """
        ...


    @staticmethod
    def isInterface(mod: int) -> bool:
        """
        Return `True` if the integer argument includes the
        `interface` modifier, `False` otherwise.

        Arguments
        - mod: a set of modifiers

        Returns
        - `True` if `mod` includes the
        `interface` modifier; `False` otherwise.
        """
        ...


    @staticmethod
    def isAbstract(mod: int) -> bool:
        """
        Return `True` if the integer argument includes the
        `abstract` modifier, `False` otherwise.

        Arguments
        - mod: a set of modifiers

        Returns
        - `True` if `mod` includes the
        `abstract` modifier; `False` otherwise.
        """
        ...


    @staticmethod
    def isStrict(mod: int) -> bool:
        """
        Return `True` if the integer argument includes the
        `strictfp` modifier, `False` otherwise.

        Arguments
        - mod: a set of modifiers

        Returns
        - `True` if `mod` includes the
        `strictfp` modifier; `False` otherwise.
        """
        ...


    @staticmethod
    def toString(mod: int) -> str:
        """
        Return a string describing the access modifier flags in
        the specified modifier. For example:
        <blockquote>```
           public final synchronized strictfp
        ```</blockquote>
        The modifier names are returned in an order consistent with the
        suggested modifier orderings given in sections 8.1.1, 8.3.1, 8.4.3, 8.8.3, and 9.1.1 of
        <cite>The Java Language Specification</cite>.
        The full modifier ordering used by this method is:
        <blockquote> `public protected private abstract static final transient
        volatile synchronized native strictfp
        interface` </blockquote>
        The `interface` modifier discussed in this class is
        not a True modifier in the Java language and it appears after
        all other modifiers listed by this method.  This method may
        return a string of modifiers that are not valid modifiers of a
        Java entity; in other words, no checking is done on the
        possible validity of the combination of modifiers represented
        by the input.
        
        Note that to perform such checking for a known kind of entity,
        such as a constructor or method, first AND the argument of
        `toString` with the appropriate mask from a method like
        .constructorModifiers or .methodModifiers.

        Arguments
        - mod: a set of modifiers

        Returns
        - a string representation of the set of modifiers
        represented by `mod`
        """
        ...


    @staticmethod
    def classModifiers() -> int:
        """
        Return an `int` value OR-ing together the source language
        modifiers that can be applied to a class.

        Returns
        - an `int` value OR-ing together the source language
        modifiers that can be applied to a class.

        Since
        - 1.7

        Unknown Tags
        - 8.1.1 Class Modifiers
        """
        ...


    @staticmethod
    def interfaceModifiers() -> int:
        """
        Return an `int` value OR-ing together the source language
        modifiers that can be applied to an interface.

        Returns
        - an `int` value OR-ing together the source language
        modifiers that can be applied to an interface.

        Since
        - 1.7

        Unknown Tags
        - 9.1.1 Interface Modifiers
        """
        ...


    @staticmethod
    def constructorModifiers() -> int:
        """
        Return an `int` value OR-ing together the source language
        modifiers that can be applied to a constructor.

        Returns
        - an `int` value OR-ing together the source language
        modifiers that can be applied to a constructor.

        Since
        - 1.7

        Unknown Tags
        - 8.8.3 Constructor Modifiers
        """
        ...


    @staticmethod
    def methodModifiers() -> int:
        """
        Return an `int` value OR-ing together the source language
        modifiers that can be applied to a method.

        Returns
        - an `int` value OR-ing together the source language
        modifiers that can be applied to a method.

        Since
        - 1.7

        Unknown Tags
        - 8.4.3 Method Modifiers
        """
        ...


    @staticmethod
    def fieldModifiers() -> int:
        """
        Return an `int` value OR-ing together the source language
        modifiers that can be applied to a field.

        Returns
        - an `int` value OR-ing together the source language
        modifiers that can be applied to a field.

        Since
        - 1.7

        Unknown Tags
        - 8.3.1 Field Modifiers
        """
        ...


    @staticmethod
    def parameterModifiers() -> int:
        """
        Return an `int` value OR-ing together the source language
        modifiers that can be applied to a parameter.

        Returns
        - an `int` value OR-ing together the source language
        modifiers that can be applied to a parameter.

        Since
        - 1.8

        Unknown Tags
        - 8.4.1 Formal Parameters
        """
        ...



"""
Python module generated from Java source file org.bukkit.event.entity.EntityEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.entity import Entity
from org.bukkit.entity import EntityType
from org.bukkit.event import Event
from org.bukkit.event.entity import *
from typing import Any, Callable, Iterable, Tuple


class EntityEvent(Event):
    """
    Represents an Entity-related event
    """

    def __init__(self, what: "Entity"):
        ...


    def getEntity(self) -> "Entity":
        """
        Returns the Entity involved in this event

        Returns
        - Entity who is involved in this event
        """
        ...


    def getEntityType(self) -> "EntityType":
        """
        Gets the EntityType of the Entity involved in this event.

        Returns
        - EntityType of the Entity involved in this event
        """
        ...



"""
Python module generated from Java source file org.bukkit.util.Vector

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.base import Preconditions
from com.google.common.primitives import Doubles
from java.util import Random
from org.bukkit import Location
from org.bukkit import World
from org.bukkit.configuration.serialization import ConfigurationSerializable
from org.bukkit.configuration.serialization import SerializableAs
from org.bukkit.util import *
from org.joml import RoundingMode
from org.joml import Vector3d
from org.joml import Vector3dc
from org.joml import Vector3f
from org.joml import Vector3fc
from org.joml import Vector3i
from org.joml import Vector3ic
from typing import Any, Callable, Iterable, Tuple


class Vector(Cloneable, ConfigurationSerializable):
    """
    Represents a mutable vector. Because the components of Vectors are mutable,
    storing Vectors long term may be dangerous if passing code modifies the
    Vector later. If you want to keep around a Vector, it may be wise to call
    `clone()` in order to get a copy.
    """

    def __init__(self):
        """
        Construct the vector with all components as 0.
        """
        ...


    def __init__(self, x: int, y: int, z: int):
        """
        Construct the vector with provided integer components.

        Arguments
        - x: X component
        - y: Y component
        - z: Z component
        """
        ...


    def __init__(self, x: float, y: float, z: float):
        """
        Construct the vector with provided double components.

        Arguments
        - x: X component
        - y: Y component
        - z: Z component
        """
        ...


    def __init__(self, x: float, y: float, z: float):
        """
        Construct the vector with provided float components.

        Arguments
        - x: X component
        - y: Y component
        - z: Z component
        """
        ...


    def add(self, vec: "Vector") -> "Vector":
        """
        Adds a vector to this one

        Arguments
        - vec: The other vector

        Returns
        - the same vector
        """
        ...


    def subtract(self, vec: "Vector") -> "Vector":
        """
        Subtracts a vector from this one.

        Arguments
        - vec: The other vector

        Returns
        - the same vector
        """
        ...


    def multiply(self, vec: "Vector") -> "Vector":
        """
        Multiplies the vector by another.

        Arguments
        - vec: The other vector

        Returns
        - the same vector
        """
        ...


    def divide(self, vec: "Vector") -> "Vector":
        """
        Divides the vector by another.

        Arguments
        - vec: The other vector

        Returns
        - the same vector
        """
        ...


    def copy(self, vec: "Vector") -> "Vector":
        """
        Copies another vector

        Arguments
        - vec: The other vector

        Returns
        - the same vector
        """
        ...


    def length(self) -> float:
        """
        Gets the magnitude of the vector, defined as sqrt(x^2+y^2+z^2). The
        value of this method is not cached and uses a costly square-root
        function, so do not repeatedly call this method to get the vector's
        magnitude. NaN will be returned if the inner result of the sqrt()
        function overflows, which will be caused if the length is too long.

        Returns
        - the magnitude
        """
        ...


    def lengthSquared(self) -> float:
        """
        Gets the magnitude of the vector squared.

        Returns
        - the magnitude
        """
        ...


    def distance(self, o: "Vector") -> float:
        """
        Get the distance between this vector and another. The value of this
        method is not cached and uses a costly square-root function, so do not
        repeatedly call this method to get the vector's magnitude. NaN will be
        returned if the inner result of the sqrt() function overflows, which
        will be caused if the distance is too long.

        Arguments
        - o: The other vector

        Returns
        - the distance
        """
        ...


    def distanceSquared(self, o: "Vector") -> float:
        """
        Get the squared distance between this vector and another.

        Arguments
        - o: The other vector

        Returns
        - the distance
        """
        ...


    def angle(self, other: "Vector") -> float:
        """
        Gets the angle between this vector and another in radians.

        Arguments
        - other: The other vector

        Returns
        - angle in radians
        """
        ...


    def midpoint(self, other: "Vector") -> "Vector":
        """
        Sets this vector to the midpoint between this vector and another.

        Arguments
        - other: The other vector

        Returns
        - this same vector (now a midpoint)
        """
        ...


    def getMidpoint(self, other: "Vector") -> "Vector":
        """
        Gets a new midpoint vector between this vector and another.

        Arguments
        - other: The other vector

        Returns
        - a new midpoint vector
        """
        ...


    def multiply(self, m: int) -> "Vector":
        """
        Performs scalar multiplication, multiplying all components with a
        scalar.

        Arguments
        - m: The factor

        Returns
        - the same vector
        """
        ...


    def multiply(self, m: float) -> "Vector":
        """
        Performs scalar multiplication, multiplying all components with a
        scalar.

        Arguments
        - m: The factor

        Returns
        - the same vector
        """
        ...


    def multiply(self, m: float) -> "Vector":
        """
        Performs scalar multiplication, multiplying all components with a
        scalar.

        Arguments
        - m: The factor

        Returns
        - the same vector
        """
        ...


    def dot(self, other: "Vector") -> float:
        """
        Calculates the dot product of this vector with another. The dot product
        is defined as x1*x2+y1*y2+z1*z2. The returned value is a scalar.

        Arguments
        - other: The other vector

        Returns
        - dot product
        """
        ...


    def crossProduct(self, o: "Vector") -> "Vector":
        """
        Calculates the cross product of this vector with another. The cross
        product is defined as:
        
        - x = y1 * z2 - y2 * z1
        - y = z1 * x2 - z2 * x1
        - z = x1 * y2 - x2 * y1

        Arguments
        - o: The other vector

        Returns
        - the same vector
        """
        ...


    def getCrossProduct(self, o: "Vector") -> "Vector":
        """
        Calculates the cross product of this vector with another without mutating
        the original. The cross product is defined as:
        
        - x = y1 * z2 - y2 * z1
        - y = z1 * x2 - z2 * x1
        - z = x1 * y2 - x2 * y1

        Arguments
        - o: The other vector

        Returns
        - a new vector
        """
        ...


    def normalize(self) -> "Vector":
        """
        Converts this vector to a unit vector (a vector with length of 1).

        Returns
        - the same vector
        """
        ...


    def zero(self) -> "Vector":
        """
        Zero this vector's components.

        Returns
        - the same vector
        """
        ...


    def isZero(self) -> bool:
        """
        Check whether or not each component of this vector is equal to 0.

        Returns
        - True if equal to zero, False if at least one component is non-zero
        """
        ...


    def isInAABB(self, min: "Vector", max: "Vector") -> bool:
        """
        Returns whether this vector is in an axis-aligned bounding box.
        
        The minimum and maximum vectors given must be truly the minimum and
        maximum X, Y and Z components.

        Arguments
        - min: Minimum vector
        - max: Maximum vector

        Returns
        - whether this vector is in the AABB
        """
        ...


    def isInSphere(self, origin: "Vector", radius: float) -> bool:
        """
        Returns whether this vector is within a sphere.

        Arguments
        - origin: Sphere origin.
        - radius: Sphere radius

        Returns
        - whether this vector is in the sphere
        """
        ...


    def isNormalized(self) -> bool:
        """
        Returns if a vector is normalized

        Returns
        - whether the vector is normalised
        """
        ...


    def rotateAroundX(self, angle: float) -> "Vector":
        """
        Rotates the vector around the x axis.
        
        This piece of math is based on the standard rotation matrix for vectors
        in three dimensional space. This matrix can be found here:
        <a href="https://en.wikipedia.org/wiki/Rotation_matrix#Basic_rotations">Rotation
        Matrix</a>.

        Arguments
        - angle: the angle to rotate the vector about. This angle is passed
        in radians

        Returns
        - the same vector
        """
        ...


    def rotateAroundY(self, angle: float) -> "Vector":
        """
        Rotates the vector around the y axis.
        
        This piece of math is based on the standard rotation matrix for vectors
        in three dimensional space. This matrix can be found here:
        <a href="https://en.wikipedia.org/wiki/Rotation_matrix#Basic_rotations">Rotation
        Matrix</a>.

        Arguments
        - angle: the angle to rotate the vector about. This angle is passed
        in radians

        Returns
        - the same vector
        """
        ...


    def rotateAroundZ(self, angle: float) -> "Vector":
        """
        Rotates the vector around the z axis
        
        This piece of math is based on the standard rotation matrix for vectors
        in three dimensional space. This matrix can be found here:
        <a href="https://en.wikipedia.org/wiki/Rotation_matrix#Basic_rotations">Rotation
        Matrix</a>.

        Arguments
        - angle: the angle to rotate the vector about. This angle is passed
        in radians

        Returns
        - the same vector
        """
        ...


    def rotateAroundAxis(self, axis: "Vector", angle: float) -> "Vector":
        """
        Rotates the vector around a given arbitrary axis in 3 dimensional space.
        
        
        Rotation will follow the general Right-Hand-Rule, which means rotation
        will be counterclockwise when the axis is pointing towards the observer.
        
        This method will always make sure the provided axis is a unit vector, to
        not modify the length of the vector when rotating. If you are experienced
        with the scaling of a non-unit axis vector, you can use
        Vector.rotateAroundNonUnitAxis(Vector, double).

        Arguments
        - axis: the axis to rotate the vector around. If the passed vector is
        not of length 1, it gets copied and normalized before using it for the
        rotation. Please use Vector.normalize() on the instance before
        passing it to this method
        - angle: the angle to rotate the vector around the axis

        Returns
        - the same vector

        Raises
        - IllegalArgumentException: if the provided axis vector instance is
        null
        """
        ...


    def rotateAroundNonUnitAxis(self, axis: "Vector", angle: float) -> "Vector":
        """
        Rotates the vector around a given arbitrary axis in 3 dimensional space.
        
        
        Rotation will follow the general Right-Hand-Rule, which means rotation
        will be counterclockwise when the axis is pointing towards the observer.
        
        Note that the vector length will change accordingly to the axis vector
        length. If the provided axis is not a unit vector, the rotated vector
        will not have its previous length. The scaled length of the resulting
        vector will be related to the axis vector. If you are not perfectly sure
        about the scaling of the vector, use
        Vector.rotateAroundAxis(Vector, double)

        Arguments
        - axis: the axis to rotate the vector around.
        - angle: the angle to rotate the vector around the axis

        Returns
        - the same vector

        Raises
        - IllegalArgumentException: if the provided axis vector instance is
        null
        """
        ...


    def getX(self) -> float:
        """
        Gets the X component.

        Returns
        - The X component.
        """
        ...


    def getBlockX(self) -> int:
        """
        Gets the floored value of the X component, indicating the block that
        this vector is contained with.

        Returns
        - block X
        """
        ...


    def getY(self) -> float:
        """
        Gets the Y component.

        Returns
        - The Y component.
        """
        ...


    def getBlockY(self) -> int:
        """
        Gets the floored value of the Y component, indicating the block that
        this vector is contained with.

        Returns
        - block y
        """
        ...


    def getZ(self) -> float:
        """
        Gets the Z component.

        Returns
        - The Z component.
        """
        ...


    def getBlockZ(self) -> int:
        """
        Gets the floored value of the Z component, indicating the block that
        this vector is contained with.

        Returns
        - block z
        """
        ...


    def setX(self, x: int) -> "Vector":
        """
        Set the X component.

        Arguments
        - x: The new X component.

        Returns
        - This vector.
        """
        ...


    def setX(self, x: float) -> "Vector":
        """
        Set the X component.

        Arguments
        - x: The new X component.

        Returns
        - This vector.
        """
        ...


    def setX(self, x: float) -> "Vector":
        """
        Set the X component.

        Arguments
        - x: The new X component.

        Returns
        - This vector.
        """
        ...


    def setY(self, y: int) -> "Vector":
        """
        Set the Y component.

        Arguments
        - y: The new Y component.

        Returns
        - This vector.
        """
        ...


    def setY(self, y: float) -> "Vector":
        """
        Set the Y component.

        Arguments
        - y: The new Y component.

        Returns
        - This vector.
        """
        ...


    def setY(self, y: float) -> "Vector":
        """
        Set the Y component.

        Arguments
        - y: The new Y component.

        Returns
        - This vector.
        """
        ...


    def setZ(self, z: int) -> "Vector":
        """
        Set the Z component.

        Arguments
        - z: The new Z component.

        Returns
        - This vector.
        """
        ...


    def setZ(self, z: float) -> "Vector":
        """
        Set the Z component.

        Arguments
        - z: The new Z component.

        Returns
        - This vector.
        """
        ...


    def setZ(self, z: float) -> "Vector":
        """
        Set the Z component.

        Arguments
        - z: The new Z component.

        Returns
        - This vector.
        """
        ...


    def equals(self, obj: "Object") -> bool:
        """
        Checks to see if two objects are equal.
        
        Only two Vectors can ever return True. This method uses a fuzzy match
        to account for floating point errors. The epsilon can be retrieved
        with epsilon.
        """
        ...


    def hashCode(self) -> int:
        """
        Returns a hash code for this vector

        Returns
        - hash code
        """
        ...


    def clone(self) -> "Vector":
        """
        Get a new vector.

        Returns
        - vector
        """
        ...


    def toString(self) -> str:
        """
        Returns this vector's components as x,y,z.
        """
        ...


    def toLocation(self, world: "World") -> "Location":
        """
        Gets a Location version of this vector with yaw and pitch being 0.

        Arguments
        - world: The world to link the location to.

        Returns
        - the location
        """
        ...


    def toLocation(self, world: "World", yaw: float, pitch: float) -> "Location":
        """
        Gets a Location version of this vector.

        Arguments
        - world: The world to link the location to.
        - yaw: The desired yaw.
        - pitch: The desired pitch.

        Returns
        - the location
        """
        ...


    def toBlockVector(self) -> "BlockVector":
        """
        Get the block vector of this vector.

        Returns
        - A block vector.
        """
        ...


    def toVector3f(self) -> "Vector3f":
        """
        Get this vector as a JOML Vector3f.

        Returns
        - the JOML vector
        """
        ...


    def toVector3d(self) -> "Vector3d":
        """
        Get this vector as a JOML Vector3d.

        Returns
        - the JOML vector
        """
        ...


    def toVector3i(self, roundingMode: int) -> "Vector3i":
        """
        Get this vector as a JOML Vector3i.

        Arguments
        - roundingMode: the RoundingMode to use for this vector's components

        Returns
        - the JOML vector
        """
        ...


    def toVector3i(self) -> "Vector3i":
        """
        Get this vector as a JOML Vector3i with its components floored.

        Returns
        - the JOML vector

        See
        - .toVector3i(int)
        """
        ...


    def checkFinite(self) -> None:
        """
        Check if each component of this Vector is finite.

        Raises
        - IllegalArgumentException: if any component is not finite
        """
        ...


    @staticmethod
    def getEpsilon() -> float:
        """
        Get the threshold used for equals().

        Returns
        - The epsilon.
        """
        ...


    @staticmethod
    def getMinimum(v1: "Vector", v2: "Vector") -> "Vector":
        """
        Gets the minimum components of two vectors.

        Arguments
        - v1: The first vector.
        - v2: The second vector.

        Returns
        - minimum
        """
        ...


    @staticmethod
    def getMaximum(v1: "Vector", v2: "Vector") -> "Vector":
        """
        Gets the maximum components of two vectors.

        Arguments
        - v1: The first vector.
        - v2: The second vector.

        Returns
        - maximum
        """
        ...


    @staticmethod
    def getRandom() -> "Vector":
        """
        Gets a random vector with components having a random value between 0
        and 1.

        Returns
        - A random vector.
        """
        ...


    @staticmethod
    def fromJOML(vector: "Vector3f") -> "Vector":
        """
        Gets a vector with components that match the provided JOML Vector3f.

        Arguments
        - vector: the vector to match

        Returns
        - the new vector
        """
        ...


    @staticmethod
    def fromJOML(vector: "Vector3d") -> "Vector":
        """
        Gets a vector with components that match the provided JOML Vector3d.

        Arguments
        - vector: the vector to match

        Returns
        - the new vector
        """
        ...


    @staticmethod
    def fromJOML(vector: "Vector3i") -> "Vector":
        """
        Gets a vector with components that match the provided JOML Vector3i.

        Arguments
        - vector: the vector to match

        Returns
        - the new vector
        """
        ...


    @staticmethod
    def fromJOML(vector: "Vector3fc") -> "Vector":
        """
        Gets a vector with components that match the provided JOML Vector3fc.

        Arguments
        - vector: the vector to match

        Returns
        - the new vector
        """
        ...


    @staticmethod
    def fromJOML(vector: "Vector3dc") -> "Vector":
        """
        Gets a vector with components that match the provided JOML Vector3dc.

        Arguments
        - vector: the vector to match

        Returns
        - the new vector
        """
        ...


    @staticmethod
    def fromJOML(vector: "Vector3ic") -> "Vector":
        """
        Gets a vector with components that match the provided JOML Vector3ic.

        Arguments
        - vector: the vector to match

        Returns
        - the new vector
        """
        ...


    def serialize(self) -> dict[str, "Object"]:
        ...


    @staticmethod
    def deserialize(args: dict[str, "Object"]) -> "Vector":
        ...



"""
Python module generated from Java source file java.io.Serializable

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.io import *
from typing import Any, Callable, Iterable, Tuple


class Serializable:
    """
    Serializability of a class is enabled by the class implementing the
    java.io.Serializable interface.
    
    <strong>Warning: Deserialization of untrusted data is inherently dangerous
    and should be avoided. Untrusted data should be carefully validated according to the
    "Serialization and Deserialization" section of the
    secure_coding_guidelines_javase Secure Coding Guidelines for Java SE.
    serialization_filter_guide Serialization Filtering describes best
    practices for defensive use of serial filters.
    </strong>
    
    Classes that do not implement this
    interface will not have any of their state serialized or
    deserialized.  All subtypes of a serializable class are themselves
    serializable.  The serialization interface has no methods or fields
    and serves only to identify the semantics of being serializable. 
    
    It is possible for subtypes of non-serializable classes to be serialized
    and deserialized. During serialization, no data will be written for the
    fields of non-serializable superclasses. During deserialization, the fields of non-serializable
    superclasses will be initialized using the no-arg constructor of the first (bottommost)
    non-serializable superclass. This constructor must be accessible to the subclass that is being
    deserialized. It is an error to declare a class Serializable if this is not
    the case; the error will be detected at runtime. A serializable subtype may
    assume responsibility for saving and restoring the state of a non-serializable
    supertype's public, protected, and (if accessible) package-access fields. See
    the <a href="/../specs/serialization/input.html#the-objectinputstream-class">
    <cite>Java Object Serialization Specification,</cite></a> section 3.1, for
    a detailed specification of the deserialization process, including handling of
    serializable and non-serializable classes. 
    
    When traversing a graph, an object may be encountered that does not
    support the Serializable interface. In this case the
    NotSerializableException will be thrown and will identify the class
    of the non-serializable object. 
    
    Classes that require special handling during the serialization and
    deserialization process must implement special methods with these exact
    signatures:
    
    <PRE>
    private void writeObject(java.io.ObjectOutputStream out)
        throws IOException
    private void readObject(java.io.ObjectInputStream in)
        throws IOException, ClassNotFoundException;
    private void readObjectNoData()
        throws ObjectStreamException;
    </PRE>
    
    The writeObject method is responsible for writing the state of the
    object for its particular class so that the corresponding
    readObject method can restore it.  The default mechanism for saving
    the Object's fields can be invoked by calling
    out.defaultWriteObject. The method does not need to concern
    itself with the state belonging to its superclasses or subclasses.
    State is saved by writing the individual fields to the
    ObjectOutputStream using the writeObject method or by using the
    methods for primitive data types supported by DataOutput.
    
    The readObject method is responsible for reading from the stream and
    restoring the classes fields. It may call in.defaultReadObject to invoke
    the default mechanism for restoring the object's non-static and
    non-transient fields.  The defaultReadObject method uses information in
    the stream to assign the fields of the object saved in the stream with the
    correspondingly named fields in the current object.  This handles the case
    when the class has evolved to add new fields. The method does not need to
    concern itself with the state belonging to its superclasses or subclasses.
    State is restored by reading data from the ObjectInputStream for
    the individual fields and making assignments to the appropriate fields
    of the object. Reading primitive data types is supported by DataInput.
    
    The readObjectNoData method is responsible for initializing the state of
    the object for its particular class in the event that the serialization
    stream does not list the given class as a superclass of the object being
    deserialized.  This may occur in cases where the receiving party uses a
    different version of the deserialized instance's class than the sending
    party, and the receiver's version extends classes that are not extended by
    the sender's version.  This may also occur if the serialization stream has
    been tampered; hence, readObjectNoData is useful for initializing
    deserialized objects properly despite a "hostile" or incomplete source
    stream.
    
    Serializable classes that need to designate an alternative object to be
    used when writing an object to the stream should implement this
    special method with the exact signature:
    
    <PRE>
    ANY-ACCESS-MODIFIER Object writeReplace() throws ObjectStreamException;
    </PRE>
    
    This writeReplace method is invoked by serialization if the method
    exists and it would be accessible from a method defined within the
    class of the object being serialized. Thus, the method can have private,
    protected and package-private access. Subclass access to this method
    follows java accessibility rules. 
    
    Classes that need to designate a replacement when an instance of it
    is read from the stream should implement this special method with the
    exact signature.
    
    <PRE>
    ANY-ACCESS-MODIFIER Object readResolve() throws ObjectStreamException;
    </PRE>
    
    This readResolve method follows the same invocation rules and
    accessibility rules as writeReplace.
    
    Enum types are all serializable and receive treatment defined by
    the <a href="/../specs/serialization/index.html"><cite>
    Java Object Serialization Specification</cite></a> during
    serialization and deserialization. Any declarations of the special
    handling methods discussed above are ignored for enum types.
    
    Record classes can implement `Serializable` and receive treatment defined
    by the <a href="/../specs/serialization/serial-arch.html#serialization-of-records">
    <cite>Java Object Serialization Specification,</cite> Section 1.13,
    "Serialization of Records"</a>. Any declarations of the special
    handling methods discussed above are ignored for record types.
    
    The serialization runtime associates with each serializable class a version
    number, called a serialVersionUID, which is used during deserialization to
    verify that the sender and receiver of a serialized object have loaded
    classes for that object that are compatible with respect to serialization.
    If the receiver has loaded a class for the object that has a different
    serialVersionUID than that of the corresponding sender's class, then
    deserialization will result in an InvalidClassException.  A
    serializable class can declare its own serialVersionUID explicitly by
    declaring a field named `"serialVersionUID"` that must be static,
    final, and of type `long`:
    
    <PRE>
    ANY-ACCESS-MODIFIER static final long serialVersionUID = 42L;
    </PRE>
    
    If a serializable class does not explicitly declare a serialVersionUID, then
    the serialization runtime will calculate a default serialVersionUID value
    for that class based on various aspects of the class, as described in the
    <a href="/../specs/serialization/index.html"><cite>Java Object Serialization
    Specification.</cite></a> This specification defines the
    serialVersionUID of an enum type to be 0L. However, it is *strongly
    recommended* that all serializable classes other than enum types explicitly declare
    serialVersionUID values, since the default serialVersionUID computation is
    highly sensitive to class details that may vary depending on compiler
    implementations, and can thus result in unexpected
    `InvalidClassException`s during deserialization.  Therefore, to
    guarantee a consistent serialVersionUID value across different java compiler
    implementations, a serializable class must declare an explicit
    serialVersionUID value.  It is also strongly advised that explicit
    serialVersionUID declarations use the `private` modifier where
    possible, since such declarations apply only to the immediately declaring
    class--serialVersionUID fields are not useful as inherited members. Array
    classes cannot declare an explicit serialVersionUID, so they always have
    the default computed value, but the requirement for matching
    serialVersionUID values is waived for array classes.

    See
    - <a href="/../specs/serialization/index.html">
         <cite>Java Object Serialization Specification</cite></a>

    Since
    - 1.1
    """





"""
Python module generated from Java source file com.google.common.base.Absent

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.base import *
from java.util import Collections
from javax.annotation import CheckForNull
from typing import Any, Callable, Iterable, Tuple


class Absent(Optional):
    """
    Implementation of an Optional not containing a reference.
    """

    def isPresent(self) -> bool:
        ...


    def get(self) -> "T":
        ...


    def or(self, defaultValue: "T") -> "T":
        ...


    def or(self, secondChoice: "Optional"["T"]) -> "Optional"["T"]:
        ...


    def or(self, supplier: "Supplier"["T"]) -> "T":
        ...


    def orNull(self) -> "T":
        ...


    def asSet(self) -> set["T"]:
        ...


    def transform(self, function: "Function"["T", "V"]) -> "Optional"["V"]:
        ...


    def equals(self, object: "Object") -> bool:
        ...


    def hashCode(self) -> int:
        ...


    def toString(self) -> str:
        ...



"""
Python module generated from Java source file com.google.common.collect.ForwardingImmutableCollection

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.collect import *
from typing import Any, Callable, Iterable, Tuple


class ForwardingImmutableCollection:
    """
    Dummy class that makes the GWT serialization policy happy. It isn't used on the server-side.

    Author(s)
    - Hayward Chan
    """





"""
Python module generated from Java source file org.bukkit.event.player.PlayerStatisticIncrementEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import Bukkit
from org.bukkit import Material
from org.bukkit import Statistic
from org.bukkit.entity import EntityType
from org.bukkit.entity import Player
from org.bukkit.event import Cancellable
from org.bukkit.event import HandlerList
from org.bukkit.event.player import *
from org.bukkit.material import MaterialData
from typing import Any, Callable, Iterable, Tuple


class PlayerStatisticIncrementEvent(PlayerEvent, Cancellable):
    """
    Called when a player statistic is incremented.
    
    This event is not called for some high frequency statistics, e.g. movement
    based statistics.
    """

    def __init__(self, player: "Player", statistic: "Statistic", initialValue: int, newValue: int):
        ...


    def __init__(self, player: "Player", statistic: "Statistic", initialValue: int, newValue: int, entityType: "EntityType"):
        ...


    def __init__(self, player: "Player", statistic: "Statistic", initialValue: int, newValue: int, material: "Material"):
        ...


    def getStatistic(self) -> "Statistic":
        """
        Gets the statistic that is being incremented.

        Returns
        - the incremented statistic
        """
        ...


    def getPreviousValue(self) -> int:
        """
        Gets the previous value of the statistic.

        Returns
        - the previous value of the statistic
        """
        ...


    def getNewValue(self) -> int:
        """
        Gets the new value of the statistic.

        Returns
        - the new value of the statistic
        """
        ...


    def getEntityType(self) -> "EntityType":
        """
        Gets the EntityType if .getStatistic() getStatistic() is an
        entity statistic otherwise returns null.

        Returns
        - the EntityType of the statistic
        """
        ...


    def getMaterial(self) -> "Material":
        """
        Gets the Material if .getStatistic() getStatistic() is a block
        or item statistic otherwise returns null.

        Returns
        - the Material of the statistic
        """
        ...


    def isCancelled(self) -> bool:
        ...


    def setCancelled(self, cancel: bool) -> None:
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...



"""
Python module generated from Java source file org.bukkit.map.MapCanvas

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.map import *
from typing import Any, Callable, Iterable, Tuple


class MapCanvas:
    """
    Represents a canvas for drawing to a map. Each canvas is associated with a
    specific MapRenderer and represents that renderer's layer on the
    map.
    """

    def getMapView(self) -> "MapView":
        """
        Get the map this canvas is attached to.

        Returns
        - The MapView this canvas is attached to.
        """
        ...


    def getCursors(self) -> "MapCursorCollection":
        """
        Get the cursor collection associated with this canvas.

        Returns
        - The MapCursorCollection associated with this canvas.
        """
        ...


    def setCursors(self, cursors: "MapCursorCollection") -> None:
        """
        Set the cursor collection associated with this canvas. This does not
        usually need to be called since a MapCursorCollection is already
        provided.

        Arguments
        - cursors: The MapCursorCollection to associate with this canvas.
        """
        ...


    def setPixelColor(self, x: int, y: int, color: "Color") -> None:
        """
        Draw a pixel to the canvas.
        
        The provided color might be converted to another color,
        which is in the map color range. This means, that
        .getPixelColor(int, int) might return another
        color than set.
        
        If null is used as color, then the color returned by
        .getBasePixelColor(int, int) is shown on the map.

        Arguments
        - x: The x coordinate, from 0 to 127.
        - y: The y coordinate, from 0 to 127.
        - color: The color.
        """
        ...


    def getPixelColor(self, x: int, y: int) -> "Color":
        """
        Get a pixel from the canvas.
        
        If no color is set at the given position for this canvas, then null is
        returned and the color returned by .getBasePixelColor(int, int)
        is shown on the map.

        Arguments
        - x: The x coordinate, from 0 to 127.
        - y: The y coordinate, from 0 to 127.

        Returns
        - The color, or null if no color is set.
        """
        ...


    def getBasePixelColor(self, x: int, y: int) -> "Color":
        """
        Get a pixel from the layers below this canvas.

        Arguments
        - x: The x coordinate, from 0 to 127.
        - y: The y coordinate, from 0 to 127.

        Returns
        - The color.
        """
        ...


    def setPixel(self, x: int, y: int, color: int) -> None:
        """
        Draw a pixel to the canvas.

        Arguments
        - x: The x coordinate, from 0 to 127.
        - y: The y coordinate, from 0 to 127.
        - color: The color. See MapPalette.

        Deprecated
        - Magic value, use .setPixelColor(int, int, Color)
        """
        ...


    def getPixel(self, x: int, y: int) -> int:
        """
        Get a pixel from the canvas.

        Arguments
        - x: The x coordinate, from 0 to 127.
        - y: The y coordinate, from 0 to 127.

        Returns
        - The color. See MapPalette.

        Deprecated
        - Magic value, use .getPixelColor(int, int)
        """
        ...


    def getBasePixel(self, x: int, y: int) -> int:
        """
        Get a pixel from the layers below this canvas.

        Arguments
        - x: The x coordinate, from 0 to 127.
        - y: The y coordinate, from 0 to 127.

        Returns
        - The color. See MapPalette.

        Deprecated
        - Magic value, use .getBasePixelColor(int, int)
        """
        ...


    def drawImage(self, x: int, y: int, image: "Image") -> None:
        """
        Draw an image to the map. The image will be clipped if necessary.

        Arguments
        - x: The x coordinate of the image.
        - y: The y coordinate of the image.
        - image: The Image to draw.
        """
        ...


    def drawText(self, x: int, y: int, font: "MapFont", text: str) -> None:
        """
        Render text to the map using fancy formatting. Newline (\n) characters
        will move down one line and return to the original column, and the text
        color can be changed using sequences such as "§12;", replacing 12 with
        the palette index of the color (see MapPalette).

        Arguments
        - x: The column to start rendering on.
        - y: The row to start rendering on.
        - font: The font to use.
        - text: The formatted text to render.
        """
        ...



"""
Python module generated from Java source file org.bukkit.event.entity.EntityTeleportEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import Location
from org.bukkit.entity import Entity
from org.bukkit.event import Cancellable
from org.bukkit.event import HandlerList
from org.bukkit.event.entity import *
from typing import Any, Callable, Iterable, Tuple


class EntityTeleportEvent(EntityEvent, Cancellable):
    """
    Thrown when a non-player entity is teleported from one location to another.
    
    This may be as a result of natural causes (Enderman, Shulker), pathfinding
    (Wolf), or commands (/teleport).
    """

    def __init__(self, what: "Entity", from: "Location", to: "Location"):
        ...


    def isCancelled(self) -> bool:
        ...


    def setCancelled(self, cancel: bool) -> None:
        ...


    def getFrom(self) -> "Location":
        """
        Gets the location that this entity moved from

        Returns
        - Location this entity moved from
        """
        ...


    def setFrom(self, from: "Location") -> None:
        """
        Sets the location that this entity moved from

        Arguments
        - from: New location this entity moved from
        """
        ...


    def getTo(self) -> "Location":
        """
        Gets the location that this entity moved to

        Returns
        - Location the entity moved to
        """
        ...


    def setTo(self, to: "Location") -> None:
        """
        Sets the location that this entity moved to

        Arguments
        - to: New Location this entity moved to
        """
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...



"""
Python module generated from Java source file net.md_5.bungee.api.chat.hover.content.Item

Java source file obtained from artifact bungeecord-chat version 1.20-R0.2-20240119.213604-65

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from net.md_5.bungee.api.chat import HoverEvent
from net.md_5.bungee.api.chat import ItemTag
from net.md_5.bungee.api.chat.hover.content import *
from typing import Any, Callable, Iterable, Tuple


class Item(Content):

    def requiredAction(self) -> "HoverEvent.Action":
        ...



"""
Python module generated from Java source file org.bukkit.block.Bell

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.block import *
from org.bukkit.entity import Entity
from org.bukkit.event.block import BellRingEvent
from typing import Any, Callable, Iterable, Tuple


class Bell(TileState):
    """
    Represents a captured state of Bell.
    """

    def ring(self, entity: "Entity", direction: "BlockFace") -> bool:
        """
        Ring this bell. This will call a BellRingEvent.

        Arguments
        - entity: the entity ringing the bell
        - direction: the direction from which the bell was rung or null to
        ring in the direction that the bell is facing

        Returns
        - True if rung successfully, False if the event was cancelled
        """
        ...


    def ring(self, entity: "Entity") -> bool:
        """
        Ring this bell in the direction that the bell is facing. This will call a
        BellRingEvent.

        Arguments
        - entity: the entity ringing the bell

        Returns
        - True if rung successfully, False if the event was cancelled
        """
        ...


    def ring(self, direction: "BlockFace") -> bool:
        """
        Ring this bell. This will call a BellRingEvent.

        Arguments
        - direction: the direction from which the bell was rung or null to
        ring in the direction that the bell is facing

        Returns
        - True if rung successfully, False if the event was cancelled
        """
        ...


    def ring(self) -> bool:
        """
        Ring this bell in the direction that the bell is facing. This will call a
        BellRingEvent.

        Returns
        - True if rung successfully, False if the event was cancelled
        """
        ...


    def isShaking(self) -> bool:
        """
        Check whether or not this bell is shaking. A bell is considered to be
        shaking if it was recently rung.
        
        A bell will typically shake for 50 ticks.

        Returns
        - True if shaking, False otherwise
        """
        ...


    def getShakingTicks(self) -> int:
        """
        Get the amount of ticks since this bell has been shaking, or 0 if the
        bell is not currently shaking.
        
        A bell will typically shake for 50 ticks.

        Returns
        - the time in ticks since the bell was rung, or 0 if not shaking
        """
        ...


    def isResonating(self) -> bool:
        """
        Check whether or not this bell is resonating. A bell is considered to be
        resonating if .isShaking() while shaking, raiders were detected
        in the area and are ready to be highlighted to nearby players.
        
        A bell will typically resonate for 40 ticks.

        Returns
        - True if resonating, False otherwise
        """
        ...


    def getResonatingTicks(self) -> int:
        """
        Get the amount of ticks since this bell has been resonating, or 0 if the
        bell is not currently resonating.
        
        A bell will typically resonate for 40 ticks.

        Returns
        - the time in ticks since the bell has been resonating, or 0 if not
        resonating
        """
        ...



"""
Python module generated from Java source file org.bukkit.event.inventory.SmithItemEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.event.inventory import *
from org.bukkit.inventory import InventoryView
from org.bukkit.inventory import SmithingInventory
from typing import Any, Callable, Iterable, Tuple


class SmithItemEvent(InventoryClickEvent):
    """
    Called when the recipe of an Item is completed inside a smithing table.
    """

    def __init__(self, view: "InventoryView", type: "InventoryType.SlotType", slot: int, click: "ClickType", action: "InventoryAction"):
        ...


    def __init__(self, view: "InventoryView", type: "InventoryType.SlotType", slot: int, click: "ClickType", action: "InventoryAction", key: int):
        ...


    def getInventory(self) -> "SmithingInventory":
        ...



"""
Python module generated from Java source file java.security.MessageDigest

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.io import ByteArrayOutputStream
from java.io import PrintStream
from java.security import *
from java.util import *
from javax.crypto import SecretKey
from sun.security.jca import GetInstance
from sun.security.util import Debug
from sun.security.util import MessageDigestSpi2
from typing import Any, Callable, Iterable, Tuple


class MessageDigest(MessageDigestSpi):

    @staticmethod
    def getInstance(algorithm: str) -> "MessageDigest":
        """
        Returns a MessageDigest object that implements the specified digest
        algorithm.
        
         This method traverses the list of registered security Providers,
        starting with the most preferred Provider.
        A new MessageDigest object encapsulating the
        MessageDigestSpi implementation from the first
        Provider that supports the specified algorithm is returned.
        
         Note that the list of registered providers may be retrieved via
        the Security.getProviders() Security.getProviders() method.

        Arguments
        - algorithm: the name of the algorithm requested.
        See the MessageDigest section in the <a href=
        "/../specs/security/standard-names.html#messagedigest-algorithms">
        Java Security Standard Algorithm Names Specification</a>
        for information about standard algorithm names.

        Returns
        - a `MessageDigest` object that implements the
                specified algorithm

        Raises
        - NoSuchAlgorithmException: if no `Provider` supports a
                `MessageDigestSpi` implementation for the
                specified algorithm
        - NullPointerException: if `algorithm` is `null`

        See
        - Provider

        Unknown Tags
        - The JDK Reference Implementation additionally uses the
        `jdk.security.provider.preferred`
        Security.getProperty(String) Security property to determine
        the preferred provider order for the specified algorithm. This
        may be different than the order of providers returned by
        Security.getProviders() Security.getProviders().
        """
        ...


    @staticmethod
    def getInstance(algorithm: str, provider: str) -> "MessageDigest":
        """
        Returns a MessageDigest object that implements the specified digest
        algorithm.
        
         A new MessageDigest object encapsulating the
        MessageDigestSpi implementation from the specified provider
        is returned.  The specified provider must be registered
        in the security provider list.
        
         Note that the list of registered providers may be retrieved via
        the Security.getProviders() Security.getProviders() method.

        Arguments
        - algorithm: the name of the algorithm requested.
        See the MessageDigest section in the <a href=
        "/../specs/security/standard-names.html#messagedigest-algorithms">
        Java Security Standard Algorithm Names Specification</a>
        for information about standard algorithm names.
        - provider: the name of the provider.

        Returns
        - a `MessageDigest` object that implements the
                specified algorithm

        Raises
        - IllegalArgumentException: if the provider name is `null`
                or empty
        - NoSuchAlgorithmException: if a `MessageDigestSpi`
                implementation for the specified algorithm is not
                available from the specified provider
        - NoSuchProviderException: if the specified provider is not
                registered in the security provider list
        - NullPointerException: if `algorithm` is `null`

        See
        - Provider
        """
        ...


    @staticmethod
    def getInstance(algorithm: str, provider: "Provider") -> "MessageDigest":
        """
        Returns a MessageDigest object that implements the specified digest
        algorithm.
        
         A new MessageDigest object encapsulating the
        MessageDigestSpi implementation from the specified Provider
        object is returned.  Note that the specified Provider object
        does not have to be registered in the provider list.

        Arguments
        - algorithm: the name of the algorithm requested.
        See the MessageDigest section in the <a href=
        "/../specs/security/standard-names.html#messagedigest-algorithms">
        Java Security Standard Algorithm Names Specification</a>
        for information about standard algorithm names.
        - provider: the provider.

        Returns
        - a `MessageDigest` object that implements the
                specified algorithm

        Raises
        - IllegalArgumentException: if the specified provider is
                `null`
        - NoSuchAlgorithmException: if a `MessageDigestSpi`
                implementation for the specified algorithm is not available
                from the specified `Provider` object
        - NullPointerException: if `algorithm` is `null`

        See
        - Provider

        Since
        - 1.4
        """
        ...


    def getProvider(self) -> "Provider":
        """
        Returns the provider of this message digest object.

        Returns
        - the provider of this message digest object
        """
        ...


    def update(self, input: int) -> None:
        """
        Updates the digest using the specified byte.

        Arguments
        - input: the byte with which to update the digest.
        """
        ...


    def update(self, input: list[int], offset: int, len: int) -> None:
        """
        Updates the digest using the specified array of bytes, starting
        at the specified offset.

        Arguments
        - input: the array of bytes.
        - offset: the offset to start from in the array of bytes.
        - len: the number of bytes to use, starting at
        `offset`.
        """
        ...


    def update(self, input: list[int]) -> None:
        """
        Updates the digest using the specified array of bytes.

        Arguments
        - input: the array of bytes.
        """
        ...


    def update(self, input: "ByteBuffer") -> None:
        """
        Update the digest using the specified ByteBuffer. The digest is
        updated using the `input.remaining()` bytes starting
        at `input.position()`.
        Upon return, the buffer's position will be equal to its limit;
        its limit will not have changed.

        Arguments
        - input: the ByteBuffer

        Since
        - 1.5
        """
        ...


    def digest(self) -> list[int]:
        """
        Completes the hash computation by performing final operations
        such as padding. The digest is reset after this call is made.

        Returns
        - the array of bytes for the resulting hash value.
        """
        ...


    def digest(self, buf: list[int], offset: int, len: int) -> int:
        """
        Completes the hash computation by performing final operations
        such as padding. The digest is reset after this call is made.

        Arguments
        - buf: output buffer for the computed digest
        - offset: offset into the output buffer to begin storing the digest
        - len: number of bytes within buf allotted for the digest

        Returns
        - the number of bytes placed into `buf`

        Raises
        - DigestException: if an error occurs.
        """
        ...


    def digest(self, input: list[int]) -> list[int]:
        """
        Performs a final update on the digest using the specified array
        of bytes, then completes the digest computation. That is, this
        method first calls .update(byte[]) update(input),
        passing the *input* array to the `update` method,
        then calls .digest() digest().

        Arguments
        - input: the input to be updated before the digest is
        completed.

        Returns
        - the array of bytes for the resulting hash value.
        """
        ...


    def toString(self) -> str:
        """
        Returns a string representation of this message digest object.
        """
        ...


    @staticmethod
    def isEqual(digesta: list[int], digestb: list[int]) -> bool:
        """
        Compares two digests for equality. Two digests are equal if they have
        the same length and all bytes at corresponding positions are equal.

        Arguments
        - digesta: one of the digests to compare.
        - digestb: the other digest to compare.

        Returns
        - True if the digests are equal, False otherwise.

        Unknown Tags
        - All bytes in `digesta` are examined to determine equality.
        The calculation time depends only on the length of `digesta`.
        It does not depend on the length of `digestb` or the contents
        of `digesta` and `digestb`.
        """
        ...


    def reset(self) -> None:
        """
        Resets the digest for further use.
        """
        ...


    def getAlgorithm(self) -> str:
        """
        Returns a string that identifies the algorithm, independent of
        implementation details. The name should be a standard
        Java Security name (such as "SHA-256").
        See the MessageDigest section in the <a href=
        "/../specs/security/standard-names.html#messagedigest-algorithms">
        Java Security Standard Algorithm Names Specification</a>
        for information about standard algorithm names.

        Returns
        - the name of the algorithm
        """
        ...


    def getDigestLength(self) -> int:
        """
        Returns the length of the digest in bytes, or 0 if this operation is
        not supported by the provider and the implementation is not cloneable.

        Returns
        - the digest length in bytes, or 0 if this operation is not
        supported by the provider and the implementation is not cloneable.

        Since
        - 1.2
        """
        ...


    def clone(self) -> "Object":
        """
        Returns a clone if the implementation is cloneable.

        Returns
        - a clone if the implementation is cloneable.

        Raises
        - CloneNotSupportedException: if this is called on an
        implementation that does not support `Cloneable`.
        """
        ...



"""
Python module generated from Java source file java.io.Flushable

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.io import *
from java.io import IOException
from typing import Any, Callable, Iterable, Tuple


class Flushable:
    """
    A `Flushable` is a destination of data that can be flushed.  The
    flush method is invoked to write any buffered output to the underlying
    stream.

    Since
    - 1.5
    """

    def flush(self) -> None:
        """
        Flushes this stream by writing any buffered output to the underlying
        stream.

        Raises
        - IOException: If an I/O error occurs
        """
        ...



"""
Python module generated from Java source file java.nio.file.OpenOption

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.nio.file import *
from typing import Any, Callable, Iterable, Tuple


class OpenOption:





from .HangingPlaceEvent import HangingPlaceEvent
from .HangingEvent import HangingEvent
from .HangingBreakEvent import HangingBreakEvent
from .HangingBreakByEntityEvent import HangingBreakByEntityEvent


from .ConsumableEffect import ConsumableEffect
from .ConsumableApplyEffects import ConsumableApplyEffects
from .ConsumableClearEffects import ConsumableClearEffects
from .ConsumableTeleportRandomly import ConsumableTeleportRandomly
from .ConsumablePlaySound import ConsumablePlaySound
from .ConsumableRemoveEffect import ConsumableRemoveEffect


"""
Python module generated from Java source file com.google.common.io.InsecureRecursiveDeleteException

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.io import *
from com.google.j2objc.annotations import J2ObjCIncompatible
from java.nio.file import FileSystemException
from java.nio.file import SecureDirectoryStream
from javax.annotation import CheckForNull
from typing import Any, Callable, Iterable, Tuple


class InsecureRecursiveDeleteException(FileSystemException):
    """
    Exception indicating that a recursive delete can't be performed because the file system does not
    have the support necessary to guarantee that it is not vulnerable to race conditions that would
    allow it to delete files and directories outside of the directory being deleted (i.e., SecureDirectoryStream is not supported).
    
    RecursiveDeleteOption.ALLOW_INSECURE can be used to force the recursive delete method
    to proceed anyway.

    Author(s)
    - Colin Decker

    Since
    - 21.0
    """

    def __init__(self, file: str):
        ...



"""
Python module generated from Java source file com.google.common.graph.Graph

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import Beta
from com.google.common.graph import *
from com.google.errorprone.annotations import DoNotMock
from javax.annotation import CheckForNull
from typing import Any, Callable, Iterable, Tuple


class Graph(BaseGraph):
    """
    An interface for <a
    href="https://en.wikipedia.org/wiki/Graph_(discrete_mathematics)">graph</a>-structured data,
    whose edges are anonymous entities with no identity or information of their own.
    
    A graph is composed of a set of nodes and a set of edges connecting pairs of nodes.
    
    There are three primary interfaces provided to represent graphs. In order of increasing
    complexity they are: Graph, ValueGraph, and Network. You should generally
    prefer the simplest interface that satisfies your use case. See the <a
    href="https://github.com/google/guava/wiki/GraphsExplained#choosing-the-right-graph-type">
    "Choosing the right graph type"</a> section of the Guava User Guide for more details.
    
    <h3>Capabilities</h3>
    
    `Graph` supports the following use cases (<a
    href="https://github.com/google/guava/wiki/GraphsExplained#definitions">definitions of
    terms</a>):
    
    
      - directed graphs
      - undirected graphs
      - graphs that do/don't allow self-loops
      - graphs whose nodes/edges are insertion-ordered, sorted, or unordered
    
    
    `Graph` explicitly does not support parallel edges, and forbids implementations or
    extensions with parallel edges. If you need parallel edges, use Network.
    
    <h3>Building a `Graph`</h3>
    
    The implementation classes that `common.graph` provides are not public, by design. To
    create an instance of one of the built-in implementations of `Graph`, use the GraphBuilder class:
    
    ````MutableGraph<Integer> graph = GraphBuilder.undirected().build();````
    
    GraphBuilder.build() returns an instance of MutableGraph, which is a subtype
    of `Graph` that provides methods for adding and removing nodes and edges. If you do not
    need to mutate a graph (e.g. if you write a method than runs a read-only algorithm on the graph),
    you should use the non-mutating Graph interface, or an ImmutableGraph.
    
    You can create an immutable copy of an existing `Graph` using ImmutableGraph.copyOf(Graph):
    
    ````ImmutableGraph<Integer> immutableGraph = ImmutableGraph.copyOf(graph);````
    
    Instances of ImmutableGraph do not implement MutableGraph (obviously!) and are
    contractually guaranteed to be unmodifiable and thread-safe.
    
    The Guava User Guide has <a
    href="https://github.com/google/guava/wiki/GraphsExplained#building-graph-instances">more
    information on (and examples of) building graphs</a>.
    
    <h3>Additional documentation</h3>
    
    See the Guava User Guide for the `common.graph` package (<a
    href="https://github.com/google/guava/wiki/GraphsExplained">"Graphs Explained"</a>) for
    additional documentation, including:
    
    
      - <a
          href="https://github.com/google/guava/wiki/GraphsExplained#equals-hashcode-and-graph-equivalence">
          `equals()`, `hashCode()`, and graph equivalence</a>
      - <a href="https://github.com/google/guava/wiki/GraphsExplained#synchronization">
          Synchronization policy</a>
      - <a href="https://github.com/google/guava/wiki/GraphsExplained#notes-for-implementors">Notes
          for implementors</a>
    
    
    Type `<N>`: Node parameter type

    Author(s)
    - Joshua O'Madadhain

    Since
    - 20.0
    """

    def nodes(self) -> set["N"]:
        """
        Returns all nodes in this graph, in the order specified by .nodeOrder().
        """
        ...


    def edges(self) -> set["EndpointPair"["N"]]:
        """
        Returns all edges in this graph.
        """
        ...


    def isDirected(self) -> bool:
        """
        Returns True if the edges in this graph are directed. Directed edges connect a EndpointPair.source() source node to a EndpointPair.target() target node, while
        undirected edges connect a pair of nodes to each other.
        """
        ...


    def allowsSelfLoops(self) -> bool:
        """
        Returns True if this graph allows self-loops (edges that connect a node to itself). Attempting
        to add a self-loop to a graph that does not allow them will throw an IllegalArgumentException.
        """
        ...


    def nodeOrder(self) -> "ElementOrder"["N"]:
        """
        Returns the order of iteration for the elements of .nodes().
        """
        ...


    def incidentEdgeOrder(self) -> "ElementOrder"["N"]:
        """
        Returns an ElementOrder that specifies the order of iteration for the elements of
        .edges(), .adjacentNodes(Object), .predecessors(Object), .successors(Object) and .incidentEdges(Object).

        Since
        - 29.0
        """
        ...


    def adjacentNodes(self, node: "N") -> set["N"]:
        """
        Returns a live view of the nodes which have an incident edge in common with `node` in
        this graph.
        
        This is equal to the union of .predecessors(Object) and .successors(Object).
        
        If `node` is removed from the graph after this method is called, the `Set`
        `view` returned by this method will be invalidated, and will throw `IllegalStateException` if it is accessed in any way, with the following exceptions:
        
        
          - `view.equals(view)` evaluates to `True` (but any other `equals()` expression
              involving `view` will throw)
          - `hashCode()` does not throw
          - if `node` is re-added to the graph after having been removed, `view`'s
              behavior is undefined

        Raises
        - IllegalArgumentException: if `node` is not an element of this graph
        """
        ...


    def predecessors(self, node: "N") -> set["N"]:
        """
        Returns a live view of all nodes in this graph adjacent to `node` which can be reached by
        traversing `node`'s incoming edges *against* the direction (if any) of the edge.
        
        In an undirected graph, this is equivalent to .adjacentNodes(Object).
        
        If `node` is removed from the graph after this method is called, the `Set`
        `view` returned by this method will be invalidated, and will throw `IllegalStateException` if it is accessed in any way, with the following exceptions:
        
        
          - `view.equals(view)` evaluates to `True` (but any other `equals()` expression
              involving `view` will throw)
          - `hashCode()` does not throw
          - if `node` is re-added to the graph after having been removed, `view`'s
              behavior is undefined

        Raises
        - IllegalArgumentException: if `node` is not an element of this graph
        """
        ...


    def successors(self, node: "N") -> set["N"]:
        """
        Returns a live view of all nodes in this graph adjacent to `node` which can be reached by
        traversing `node`'s outgoing edges in the direction (if any) of the edge.
        
        In an undirected graph, this is equivalent to .adjacentNodes(Object).
        
        This is *not* the same as "all nodes reachable from `node` by following outgoing
        edges". For that functionality, see Graphs.reachableNodes(Graph, Object).
        
        If `node` is removed from the graph after this method is called, the `Set`
        `view` returned by this method will be invalidated, and will throw `IllegalStateException` if it is accessed in any way, with the following exceptions:
        
        
          - `view.equals(view)` evaluates to `True` (but any other `equals()` expression
              involving `view` will throw)
          - `hashCode()` does not throw
          - if `node` is re-added to the graph after having been removed, `view`'s
              behavior is undefined

        Raises
        - IllegalArgumentException: if `node` is not an element of this graph
        """
        ...


    def incidentEdges(self, node: "N") -> set["EndpointPair"["N"]]:
        """
        Returns a live view of the edges in this graph whose endpoints include `node`.
        
        This is equal to the union of incoming and outgoing edges.
        
        If `node` is removed from the graph after this method is called, the `Set`
        `view` returned by this method will be invalidated, and will throw `IllegalStateException` if it is accessed in any way, with the following exceptions:
        
        
          - `view.equals(view)` evaluates to `True` (but any other `equals()` expression
              involving `view` will throw)
          - `hashCode()` does not throw
          - if `node` is re-added to the graph after having been removed, `view`'s
              behavior is undefined

        Raises
        - IllegalArgumentException: if `node` is not an element of this graph

        Since
        - 24.0
        """
        ...


    def degree(self, node: "N") -> int:
        """
        Returns the count of `node`'s incident edges, counting self-loops twice (equivalently,
        the number of times an edge touches `node`).
        
        For directed graphs, this is equal to `inDegree(node) + outDegree(node)`.
        
        For undirected graphs, this is equal to `incidentEdges(node).size()` + (number of
        self-loops incident to `node`).
        
        If the count is greater than `Integer.MAX_VALUE`, returns `Integer.MAX_VALUE`.

        Raises
        - IllegalArgumentException: if `node` is not an element of this graph
        """
        ...


    def inDegree(self, node: "N") -> int:
        """
        Returns the count of `node`'s incoming edges (equal to `predecessors(node).size()`)
        in a directed graph. In an undirected graph, returns the .degree(Object).
        
        If the count is greater than `Integer.MAX_VALUE`, returns `Integer.MAX_VALUE`.

        Raises
        - IllegalArgumentException: if `node` is not an element of this graph
        """
        ...


    def outDegree(self, node: "N") -> int:
        """
        Returns the count of `node`'s outgoing edges (equal to `successors(node).size()`)
        in a directed graph. In an undirected graph, returns the .degree(Object).
        
        If the count is greater than `Integer.MAX_VALUE`, returns `Integer.MAX_VALUE`.

        Raises
        - IllegalArgumentException: if `node` is not an element of this graph
        """
        ...


    def hasEdgeConnecting(self, nodeU: "N", nodeV: "N") -> bool:
        """
        Returns True if there is an edge that directly connects `nodeU` to `nodeV`. This is
        equivalent to `nodes().contains(nodeU) && successors(nodeU).contains(nodeV)`.
        
        In an undirected graph, this is equal to `hasEdgeConnecting(nodeV, nodeU)`.

        Since
        - 23.0
        """
        ...


    def hasEdgeConnecting(self, endpoints: "EndpointPair"["N"]) -> bool:
        """
        Returns True if there is an edge that directly connects `endpoints` (in the order, if
        any, specified by `endpoints`). This is equivalent to `edges().contains(endpoints)`.
        
        Unlike the other `EndpointPair`-accepting methods, this method does not throw if the
        endpoints are unordered and the graph is directed; it simply returns `False`. This is for
        consistency with the behavior of Collection.contains(Object) (which does not generally
        throw if the object cannot be present in the collection), and the desire to have this method's
        behavior be compatible with `edges().contains(endpoints)`.

        Since
        - 27.1
        """
        ...


    def equals(self, object: "Object") -> bool:
        """
        Returns `True` iff `object` is a Graph that has the same elements and the
        same structural relationships as those in this graph.
        
        Thus, two graphs A and B are equal if **all** of the following are True:
        
        
          - A and B have equal .isDirected() directedness.
          - A and B have equal .nodes() node sets.
          - A and B have equal .edges() edge sets.
        
        
        Graph properties besides .isDirected() directedness do **not** affect equality.
        For example, two graphs may be considered equal even if one allows self-loops and the other
        doesn't. Additionally, the order in which nodes or edges are added to the graph, and the order
        in which they are iterated over, are irrelevant.
        
        A reference implementation of this is provided by AbstractGraph.equals(Object).
        """
        ...


    def hashCode(self) -> int:
        """
        Returns the hash code for this graph. The hash code of a graph is defined as the hash code of
        the set returned by .edges().
        
        A reference implementation of this is provided by AbstractGraph.hashCode().
        """
        ...



"""
Python module generated from Java source file org.yaml.snakeyaml.util.UriEncoder

Java source file obtained from artifact snakeyaml version 2.2

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.io import UnsupportedEncodingException
from java.net import URLDecoder
from java.nio.charset import CharacterCodingException
from java.nio.charset import CharsetDecoder
from java.nio.charset import CodingErrorAction
from java.nio.charset import StandardCharsets
from org.yaml.snakeyaml.error import YAMLException
from org.yaml.snakeyaml.external.com.google.gdata.util.common.base import Escaper
from org.yaml.snakeyaml.external.com.google.gdata.util.common.base import PercentEscaper
from org.yaml.snakeyaml.util import *
from typing import Any, Callable, Iterable, Tuple


class UriEncoder:

    @staticmethod
    def encode(uri: str) -> str:
        """
        Escape special characters with '%'

        Arguments
        - uri: URI to be escaped

        Returns
        - encoded URI
        """
        ...


    @staticmethod
    def decode(buff: "ByteBuffer") -> str:
        """
        Decode '%'-escaped characters. Decoding fails in case of invalid UTF-8

        Arguments
        - buff: data to decode

        Returns
        - decoded data

        Raises
        - CharacterCodingException: if cannot be decoded
        """
        ...


    @staticmethod
    def decode(buff: str) -> str:
        ...



"""
Python module generated from Java source file org.yaml.snakeyaml.resolver.Resolver

Java source file obtained from artifact snakeyaml version 2.2

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util.regex import Pattern
from org.yaml.snakeyaml.nodes import NodeId
from org.yaml.snakeyaml.nodes import Tag
from org.yaml.snakeyaml.resolver import *
from typing import Any, Callable, Iterable, Tuple


class Resolver:
    """
    Resolver tries to detect a type by content (when the tag is implicit)
    """

    BOOL = Pattern.compile("^(?:yes|Yes|YES|no|No|NO|true|True|TRUE|false|False|FALSE|on|On|ON|off|Off|OFF)$")
    FLOAT = Pattern.compile(// (base 10)
    "^(" + "[-+]?(?:[0-9][0-9_]*)\\.[0-9_]*(?:[eE][-+]?[0-9]+)?" + // (base 10, scientific notation without .)
    "|[-+]?(?:[0-9][0-9_]*)(?:[eE][-+]?[0-9]+)" + // (base 10, starting with .)
    "|[-+]?\\.[0-9_]+(?:[eE][-+]?[0-9]+)?" + // (base 60)
    "|[-+]?[0-9][0-9_]*(?::[0-5]?[0-9])+\\.[0-9_]*" + "|[-+]?\\.(?:inf|Inf|INF)" + "|\\.(?:nan|NaN|NAN)" + ")$")
    """
    The regular expression is taken from the 1.2 specification but '_'s are added to keep backwards
    compatibility
    """
    INT = Pattern.compile(// (base 2)
    "^(?:" + "[-+]?0b_*[0-1][0-1_]*" + // (base 8)
    "|[-+]?0_*[0-7][0-7_]*" + // (base 10)
    "|[-+]?(?:0|[1-9][0-9_]*)" + // (base 16)
    "|[-+]?0x_*[0-9a-fA-F][0-9a-fA-F_]*" + // (base 60)
    "|[-+]?[1-9][0-9_]*(?::[0-5]?[0-9])+" + ")$")
    MERGE = Pattern.compile("^(?:<<)$")
    NULL = Pattern.compile("^(?:~|null|Null|NULL| )$")
    EMPTY = Pattern.compile("^$")
    TIMESTAMP = Pattern.compile("^(?:[0-9][0-9][0-9][0-9]-[0-9][0-9]-[0-9][0-9]|[0-9][0-9][0-9][0-9]-[0-9][0-9]?-[0-9][0-9]?(?:[Tt]|[ \t]+)[0-9][0-9]?:[0-9][0-9]:[0-9][0-9](?:\\.[0-9]*)?(?:[ \t]*(?:Z|[-+][0-9][0-9]?(?::[0-9][0-9])?))?)$")
    VALUE = Pattern.compile("^(?:=)$")
    YAML = Pattern.compile("^(?:!|&|\\*)$")


    def __init__(self):
        ...


    def addImplicitResolver(self, tag: "Tag", regexp: "Pattern", first: str) -> None:
        ...


    def addImplicitResolver(self, tag: "Tag", regexp: "Pattern", first: str, limit: int) -> None:
        """
        Add a resolver to resolve a value that matches the provided regular expression to the provided
        tag

        Arguments
        - tag: - the Tag to assign when the value matches
        - regexp: - the RE which is applied for every value
        - first: - the possible first characters (this is merely for performance improvement) to
               skip RE evaluation to gain time
        - limit: - the limit of the value to analyze. The limit is here only to fight the DoS
               attack when huge values are provided, and it may lead to slow pattern evaluation
        """
        ...


    def resolve(self, kind: "NodeId", value: str, implicit: bool) -> "Tag":
        ...



"""
Python module generated from Java source file org.bukkit.plugin.java.LibraryLoader

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.io import File
from java.net import MalformedURLException
from java.net import URL
from java.net import URLClassLoader
from java.util import Arrays
from org.apache.maven.repository.internal import MavenRepositorySystemUtils
from org.bukkit.plugin import PluginDescriptionFile
from org.bukkit.plugin.java import *
from org.eclipse.aether import DefaultRepositorySystemSession
from org.eclipse.aether import RepositorySystem
from org.eclipse.aether.artifact import Artifact
from org.eclipse.aether.artifact import DefaultArtifact
from org.eclipse.aether.collection import CollectRequest
from org.eclipse.aether.connector.basic import BasicRepositoryConnectorFactory
from org.eclipse.aether.graph import Dependency
from org.eclipse.aether.impl import DefaultServiceLocator
from org.eclipse.aether.repository import LocalRepository
from org.eclipse.aether.repository import RemoteRepository
from org.eclipse.aether.repository import RepositoryPolicy
from org.eclipse.aether.resolution import ArtifactResult
from org.eclipse.aether.resolution import DependencyRequest
from org.eclipse.aether.resolution import DependencyResolutionException
from org.eclipse.aether.resolution import DependencyResult
from org.eclipse.aether.spi.connector import RepositoryConnectorFactory
from org.eclipse.aether.spi.connector.transport import TransporterFactory
from org.eclipse.aether.transfer import AbstractTransferListener
from org.eclipse.aether.transfer import TransferCancelledException
from org.eclipse.aether.transfer import TransferEvent
from org.eclipse.aether.transport.http import HttpTransporterFactory
from typing import Any, Callable, Iterable, Tuple


class LibraryLoader:

    def __init__(self, logger: "Logger"):
        ...


    def createLoader(self, desc: "PluginDescriptionFile") -> "ClassLoader":
        ...



"""
Python module generated from Java source file org.yaml.snakeyaml.scanner.ScannerImpl

Java source file obtained from artifact snakeyaml version 2.2

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.nio.charset import CharacterCodingException
from java.util import Iterator
from java.util.regex import Pattern
from org.yaml.snakeyaml import DumperOptions
from org.yaml.snakeyaml import LoaderOptions
from org.yaml.snakeyaml.comments import CommentType
from org.yaml.snakeyaml.error import Mark
from org.yaml.snakeyaml.error import YAMLException
from org.yaml.snakeyaml.reader import StreamReader
from org.yaml.snakeyaml.scanner import *
from org.yaml.snakeyaml.tokens import AliasToken
from org.yaml.snakeyaml.tokens import AnchorToken
from org.yaml.snakeyaml.tokens import BlockEndToken
from org.yaml.snakeyaml.tokens import BlockEntryToken
from org.yaml.snakeyaml.tokens import BlockMappingStartToken
from org.yaml.snakeyaml.tokens import BlockSequenceStartToken
from org.yaml.snakeyaml.tokens import CommentToken
from org.yaml.snakeyaml.tokens import DirectiveToken
from org.yaml.snakeyaml.tokens import DocumentEndToken
from org.yaml.snakeyaml.tokens import DocumentStartToken
from org.yaml.snakeyaml.tokens import FlowEntryToken
from org.yaml.snakeyaml.tokens import FlowMappingEndToken
from org.yaml.snakeyaml.tokens import FlowMappingStartToken
from org.yaml.snakeyaml.tokens import FlowSequenceEndToken
from org.yaml.snakeyaml.tokens import FlowSequenceStartToken
from org.yaml.snakeyaml.tokens import KeyToken
from org.yaml.snakeyaml.tokens import ScalarToken
from org.yaml.snakeyaml.tokens import StreamEndToken
from org.yaml.snakeyaml.tokens import StreamStartToken
from org.yaml.snakeyaml.tokens import TagToken
from org.yaml.snakeyaml.tokens import TagTuple
from org.yaml.snakeyaml.tokens import Token
from org.yaml.snakeyaml.tokens import ValueToken
from org.yaml.snakeyaml.util import ArrayStack
from org.yaml.snakeyaml.util import UriEncoder
from typing import Any, Callable, Iterable, Tuple


class ScannerImpl(Scanner):
    """
    ```
    Scanner produces tokens of the following types:
    STREAM-START
    STREAM-END
    COMMENT
    DIRECTIVE(name, value)
    DOCUMENT-START
    DOCUMENT-END
    BLOCK-SEQUENCE-START
    BLOCK-MAPPING-START
    BLOCK-END
    FLOW-SEQUENCE-START
    FLOW-MAPPING-START
    FLOW-SEQUENCE-END
    FLOW-MAPPING-END
    BLOCK-ENTRY
    FLOW-ENTRY
    KEY
    VALUE
    ALIAS(value)
    ANCHOR(value)
    TAG(value)
    SCALAR(value, plain, style)
    Read comments in the Scanner code for more details.
    ```
    """

    ESCAPE_REPLACEMENTS = HashMap<Character, String>()
    """
    A mapping from an escaped character in the input stream to the string representation that they
    should be replaced with.
    
    YAML defines several common and a few uncommon escape sequences.

    See
    - <a href="http://www.yaml.org/spec/current.html.id2517668">4.1.6. Escape Sequences</a>
    """
    ESCAPE_CODES = HashMap<Character, Integer>()
    """
    A mapping from a character to a number of bytes to read-ahead for that escape sequence. These
    escape sequences are used to handle unicode escaping in the following formats, where H is a
    hexadecimal character:
    
    ```
    &#92;xHH         : escaped 8-bit Unicode character
    &#92;uHHHH       : escaped 16-bit Unicode character
    &#92;UHHHHHHHH   : escaped 32-bit Unicode character
    ```

    See
    - <a href="http://yaml.org/spec/1.1/current.html.id872840">5.6. Escape Sequences</a>
    """


    def __init__(self, reader: "StreamReader", options: "LoaderOptions"):
        ...


    def checkToken(self, *choices: Tuple["Token.ID", ...]) -> bool:
        """
        Check whether the next token is one of the given types.
        """
        ...


    def peekToken(self) -> "Token":
        """
        Return the next token, but do not delete it from the queue.
        """
        ...


    def getToken(self) -> "Token":
        """
        Return the next token, removing it from the queue.
        """
        ...


    def resetDocumentIndex(self) -> None:
        ...



"""
Python module generated from Java source file org.bukkit.TreeType

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from enum import Enum
from org.bukkit import *
from typing import Any, Callable, Iterable, Tuple


class TreeType(Enum):
    """
    Tree and organic structure types.
    """

    TREE = 0
    """
    Regular tree, no branches
    """
    BIG_TREE = 1
    """
    Regular tree, extra tall with branches
    """
    REDWOOD = 2
    """
    Redwood tree, shaped like a pine tree
    """
    TALL_REDWOOD = 3
    """
    Tall redwood tree with just a few leaves at the top
    """
    BIRCH = 4
    """
    Birch tree
    """
    JUNGLE = 5
    """
    Standard jungle tree; 4 blocks wide and tall
    """
    SMALL_JUNGLE = 6
    """
    Smaller jungle tree; 1 block wide
    """
    COCOA_TREE = 7
    """
    Jungle tree with cocoa plants; 1 block wide
    """
    JUNGLE_BUSH = 8
    """
    Small bush that grows in the jungle
    """
    RED_MUSHROOM = 9
    """
    Big red mushroom; short and fat
    """
    BROWN_MUSHROOM = 10
    """
    Big brown mushroom; tall and umbrella-like
    """
    SWAMP = 11
    """
    Swamp tree (regular with vines on the side)
    """
    ACACIA = 12
    """
    Acacia tree.
    """
    DARK_OAK = 13
    """
    Dark Oak tree.
    """
    MEGA_REDWOOD = 14
    """
    Mega redwood tree; 4 blocks wide and tall
    """
    MEGA_PINE = 15
    """
    Mega pine tree
    """
    TALL_BIRCH = 16
    """
    Tall birch tree
    """
    CHORUS_PLANT = 17
    """
    Large plant native to The End
    """
    CRIMSON_FUNGUS = 18
    """
    Large crimson fungus native to the nether
    """
    WARPED_FUNGUS = 19
    """
    Large warped fungus native to the nether
    """
    AZALEA = 20
    """
    Tree with large roots which grows above lush caves
    """
    MANGROVE = 21
    """
    Mangrove tree
    """
    TALL_MANGROVE = 22
    """
    Tall mangrove tree
    """
    CHERRY = 23
    """
    Cherry tree
    """
    PALE_OAK = 24
    """
    Pale oak tree
    """
    PALE_OAK_CREAKING = 25
    """
    Pale oak tree with a creaking heart
    """
